/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.internal.PracticeElement"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.internal.RequestInterceptor"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.BootstrapProperties"); //$NON-NLS-1$
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants;
	var UIHelper = com.ibm.team.process.authoring.web.common.internal.UIHelper;
	var PROCESSDESCRIPTION_HEADER = "<ProcessDescription xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" rdf:about=\"\" xml:lang=\"en\"> \r\n"; //$NON-NLS-1$
	var PRACTICE_ITEM = "<practice dc:title=\"${title}\" rdf:resource=\"${resource}\"/>"; //$NON-NLS-1$
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient;
	var PracticeElement = com.ibm.team.process.authoring.web.common.internal.PracticeElement;
	var JTS_URI_ROOT = getJtsRoot();
	var JTS_PROCESS_AUTHORING_ROOT = JTS_URI_ROOT + ProcessAuthoringConstants.PROCESS_AUTHORING_URI_PART;
	var DEFAULT_SHARED_LIBRARY_URL = JTS_PROCESS_AUTHORING_ROOT + "libraries/shared"; //$NON-NLS-1$

	ProcessAuthoringClient.getPracticeElement = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: args.url,
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.getPracticeElementOwner = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: args.url,
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.getResource = function(args) {
	
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: args.url,
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.getPractice = function(args) {
		return ProcessAuthoringClient.getResource(args);
	};
	
	ProcessAuthoringClient.handleLock = function(args) {
		var pdata;
		var url;
		var lockElement = args.lockRequest ? "<locked-by/>\r\n" : " <locked-by  rdf:resource=\"\"/>\r\n"; //$NON-NLS-1$ //$NON-NLS-2$
		if (args.updatePractice) {
			pdata = "<Practice " + //$NON-NLS-1$
			"xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" " + //$NON-NLS-1$
			"xmlns:dc=\"http://purl.org/dc/terms/\" " + //$NON-NLS-1$
			"xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" " + //$NON-NLS-1$
			"rdf:about=\"\" " + //$NON-NLS-1$
			"xml:lang=\"en\"> " + //$NON-NLS-1$
			lockElement +
			"</Practice>"; //$NON-NLS-1$
			url = ProcessAuthoringClient.getJtsProcessAuthoringRoot() + "practices/" + args.itemId + "?updateType=partial&byForce=" + args.byForce; //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			pdata = PROCESSDESCRIPTION_HEADER + //$NON-NLS-1$
			lockElement +
			"</ProcessDescription>"; //$NON-NLS-1$
			url = ProcessAuthoringClient.getJtsProcessAuthoringRoot() + "descriptions/" + args.itemId + "?updateType=partial&byForce=" + args.byForce; //$NON-NLS-1$ //$NON-NLS-2$
		}
		jazz.client.xhrPut({
			url: url,
			headers: {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/xml", //$NON-NLS-1$ //$NON-NLS-2$
				"ETag": args.ETag + " undefined" //$NON-NLS-1$ //$NON-NLS-2$
			},
			handleAs: "json", //$NON-NLS-1$
			putData: pdata,
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.updatePractice = function(args) {
    	args['url'] = dojo.string.substitute("${0}practices/${1}", [ProcessAuthoringClient.getJtsProcessAuthoringRoot(), args.practiceId]); //$NON-NLS-1$ //$NON-NLS-2$
    	this.updatePracticeWithURL(args);
    };
	
	ProcessAuthoringClient.updatePracticeWithURL = function(args) {
    	var url = args.url + "?updateType=partial"; //$NON-NLS-1$
    	if(args.language) {
			url = url + "&lang=" + args.language; //$NON-NLS-1$
		}
    	jazz.client.xhrPut({
    		url: url,
    		headers: {
    			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
    			"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
    			"Accept": "application/xml", //$NON-NLS-1$ //$NON-NLS-2$
    			"ETag": args.ETag //$NON-NLS-1$
    		},
    		handleAs: "json", //$NON-NLS-1$
    		putData: args.putData,
    		error: args.error,
    		load: args.load
    	});
    };
	
	ProcessAuthoringClient.createSharedPractice = function(args) {		
		var pdata = "<Practice " + //$NON-NLS-1$
		"xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" " + //$NON-NLS-1$
		"xmlns:dc=\"http://purl.org/dc/terms/\" " + //$NON-NLS-1$
		"xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" " + //$NON-NLS-1$
		"rdf:about=\"\" " + //$NON-NLS-1$
		"xml:lang=\"en\"> " + //$NON-NLS-1$
		"<shared>true</shared>" + //$NON-NLS-1$
		"<dc:language>en</dc:language> " + //$NON-NLS-1$
		"<presentation-type>structure</presentation-type>" + //$NON-NLS-1$
		"<dc:title>" + //$NON-NLS-1$
		UIHelper.escapeXML(args.title) +
		"</dc:title> " + //$NON-NLS-1$
		"<dc:description>" + //$NON-NLS-1$
		UIHelper.escapeXML(args.description) +
		"</dc:description>" + //$NON-NLS-1$
		"<locked-by  rdf:resource=\"\"/>" + //$NON-NLS-1$
		"<presentation rdf:parseType=\"Literal\">" + //$NON-NLS-1$
		"</presentation>" + //$NON-NLS-1$
		"</Practice>"; //$NON-NLS-1$
		return jazz.client.xhrPost({
			url: DEFAULT_SHARED_LIBRARY_URL,
			headers: {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
			},
			handleAs: "xml", //$NON-NLS-1$
			postData: pdata,
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.createPrivatePractice = function(args) {
		args['processDescriptionUrl'] = ProcessAuthoringClient.getJtsProcessAuthoringRoot() + "descriptions/" + args.processDescriptionId; //$NON-NLS-1$ //$NON-NLS-2$
		this.createPrivatePracticeWithProcessDescriptionURL(args);
	}
	
	ProcessAuthoringClient.createPrivatePracticeWithProcessDescriptionURL = function(args) {
		var pdata = "<Practice " + //$NON-NLS-1$
		"xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" " + //$NON-NLS-1$
		"xmlns:dc=\"http://purl.org/dc/terms/\" " + //$NON-NLS-1$
		"xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" " + //$NON-NLS-1$
		"rdf:about=\"\" " + //$NON-NLS-1$
		"xml:lang=\"" + //$NON-NLS-1$
		args.language +
		"\"> " + //$NON-NLS-1$ //$NON-NLS-2$
		"<presentation-type>structure</presentation-type>" + //$NON-NLS-1$
		"<dc:title>" + //$NON-NLS-1$
		UIHelper.escapeXML(args.name) +
		"</dc:title> " + //$NON-NLS-1$
		"<dc:description>" + //$NON-NLS-1$
		UIHelper.escapeXML(args.description) +
		"</dc:description>" + //$NON-NLS-1$
		"<presentation rdf:parseType=\"Literal\">" + //$NON-NLS-1$
		"</presentation>" + //$NON-NLS-1$
		"</Practice>"; //$NON-NLS-1$
		jazz.client.xhrPost({
			url: args.processDescriptionUrl,
			headers: {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/xml", //$NON-NLS-1$ //$NON-NLS-2$
				"ETag": args.ETag //$NON-NLS-1$
			},
			handleAs: "json", //$NON-NLS-1$
			postData: pdata,
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.postPracticeElement = function(args) {
	
		var deferred = jazz.client.xhrPost({
			url: args.url,
			headers: {
				ETag: args.ETag
			},
			contentType: "application/xml; charset=utf-8", //$NON-NLS-1$
			handleAs: "json", //$NON-NLS-1$
			postData: args.postData,
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.updatePracticeElement = function(args) {
		var deferred = jazz.client.xhrPut({
			url: args.url,
			contentType: "application/xml; charset=utf-8", //$NON-NLS-1$
			headers: {
				Accept: "application/json", //$NON-NLS-1$
				ETag: args.ETag
			},
			
			handleAs: "json", //$NON-NLS-1$
			putData: args.putData,
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.deleteProcessDescription = function(args) {
		return jazz.client.xhrDelete({
			url: UIHelper.getProcessAuthoringRoot() + dojo.string.substitute("/project-area/${processAreaItemId}/description", args), //$NON-NLS-1$ //$NON-NLS-2$
			handleAs: "xml" //$NON-NLS-1$
		});
	};
	
	ProcessAuthoringClient.deleteProcessDescriptionById = function(args) {
		var url = args.uri ? args.uri : ProcessAuthoringClient.getJtsProcessAuthoringRoot() + dojo.string.substitute("descriptions/${processDescriptionId}", args); //$NON-NLS-1$
		jazz.client.xhrDelete({
			url: url,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.deletePractice = function(args) {
		var url = args.uri ? args.uri : ProcessAuthoringClient.getJtsProcessAuthoringRoot() + "practices/" + args.practiceId; //$NON-NLS-1$
		jazz.client.xhrDelete({
			url: url,
			headers: {
				ETag: args.ETag //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.checkProcessDescriptionForProcessArea = function(args) {
		return jazz.client.xhrHead({
			url: UIHelper.getProcessAuthoringRoot() + dojo.string.substitute("/${processAreaType}/${itemId}/description", args), //$NON-NLS-1$ 
			headers: {
				Accept: "application/xml" //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.checkAssociateSharedProcessDescriptionForProcessArea = function(args) {
		return jazz.client.xhrHead({
			url: ProcessAuthoringClient.getJtsProcessAuthoringRoot() + dojo.string.substitute("associations?target=${targetUrl}", args), //$NON-NLS-1$
			headers: {
				Accept: "application/xml" //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.checkAssociationForProcessDescription = function(args) {
		return jazz.client.xhrHead({
			url: ProcessAuthoringClient.getJtsProcessAuthoringRoot() + dojo.string.substitute("associations" + "?source=${sourceUrl}", args), //$NON-NLS-1$ //$NON-NLS-2$
			headers: {
				Accept: "application/xml" //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	//need to change field for owner to integrated url later
	ProcessAuthoringClient.createProcessDescriptionForProcessArea = function(args) {
		args.title = UIHelper.escapeXML(args.title);
		args.description = UIHelper.escapeXML(args.description);
		
		var postData = "<ProcessDescription xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" rdf:about=\"\" " + //$NON-NLS-1$ 
		"xml:lang=\"${language}\"> \r\n" +  //$NON-NLS-1$
		"<dc:title>${title}</dc:title>\r\n" + //$NON-NLS-1$
		"<dc:description>${description}</dc:description>\r\n" + //$NON-NLS-1$
		"<locked-by rdf:resource=\"\"/>\r\n" + //$NON-NLS-1$
		"</ProcessDescription>"; //$NON-NLS-1$
		postData = dojo.string.substitute(postData, args);
		var itemId = args.ownerUrl.substring(args.ownerUrl.lastIndexOf("/") + 1, args.ownerUrl.length); //$NON-NLS-1$
		var segments = args.ownerUrl.split('/'); //$NON-NLS-1$
		var serviceSegment = segments[segments.length - 2] === "team-areas" ? "/team-area/${0}" : "/project-area/${0}"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		jazz.client.xhrPost({
			url: UIHelper.getProcessAuthoringRoot() + dojo.string.substitute(serviceSegment, [itemId]), //$NON-NLS-1$
			headers: {
				Accept: "application/xml" //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			postData: postData,
			error: function(err) {
				args.errorCallback(err);
			},
			load: function(data, ioArgs) {
				var location = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
				args.successCallback(location, ioArgs.xhr.getResponseHeader("Etag")); //$NON-NLS-1$
			}
		});
	};
	
	ProcessAuthoringClient.createSharedProcessDescription = function(args) {
		// TODO: add implementation of invoking create shared process description service
		args.header = PROCESSDESCRIPTION_HEADER;
		args.title = UIHelper.escapeXML(args.title);
		args.description = UIHelper.escapeXML(args.description);
		
		var postData = "${header}" + //$NON-NLS-1$
		"<dc:title>${title}</dc:title>\r\n" + //$NON-NLS-1$
		"<dc:description>${description}</dc:description>\r\n" + //$NON-NLS-1$
		"<shared>${isPublished}</shared>\r\n" + //$NON-NLS-1$
		"<locked-by rdf:resource=\"\"/>\r\n" + //$NON-NLS-1$
		"</ProcessDescription>"; //$NON-NLS-1$
		postData = dojo.string.substitute(postData, args);
		jazz.client.xhrPost({
			url: DEFAULT_SHARED_LIBRARY_URL,
			headers: {
				'Content-Type': "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				'Accept': "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
			},
			handleAs: "xml", //$NON-NLS-1$
			postData: postData,
			error: function(err, ioArgs) {
				var licenseWarning = UIHelper.getLicenseWarning(ioArgs);
				if(licenseWarning) {
					err.headerMessage = licenseWarning;
				}
				args.errorCallback(err);
			},
			load: function(data, ioArgs) {
				var location = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
				args.successCallback(location, ioArgs.xhr.getResponseHeader("Etag")); //$NON-NLS-1$
			}
		});
	};
	
	ProcessAuthoringClient.updateProcessDescriptionPracitceList = function(args) {
		var pdata = "<ProcessDescription xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" rdf:about=\"\" xml:lang=\"${language}\"> \r\n"+  //$NON-NLS-1$
		            "<owner rdf:resource=\"${owner}\"/>\r\n";   //$NON-NLS-1$
		pdata = dojo.string.substitute(pdata, args);
		
		var practicesNum = 0;
		
		dojo.forEach(args.practiceList, function(practice) {
			pdata += dojo.string.substitute("<practice shared=\"${0}\" dc:title=\"${1}\" rdf:resource=\"${2}\"/> \r\n", [practice.shared, UIHelper.escapeXML(practice.name), practice.url]); //$NON-NLS-1$
			practicesNum++;
		}, this);
		
		if (practicesNum === 0) {
			pdata += "<practice rdf:resource=\"\" />\r\n"; //$NON-NLS-1$	
		}
		
		pdata += "</ProcessDescription>"; //$NON-NLS-1$
		jazz.client.xhrPut({
			url: ProcessAuthoringClient.getJtsProcessAuthoringRoot() + dojo.string.substitute("descriptions/${itemId}?updateType=partial&lang=${language}", args), //$NON-NLS-1$ //$NON-NLS-2$
			headers: {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"ETag": args.ETag //$NON-NLS-1$
			},
			handleAs: "text", //$NON-NLS-1$
			putData: pdata,
			error: args.error,
			load: args.load
		});
	};


	//input: title, description, owner, ETag
	ProcessAuthoringClient.updateProcessDescription = function(args) {
		var url = ProcessAuthoringClient.getJtsProcessAuthoringRoot() + dojo.string.substitute("descriptions/${itemId}", args); //$NON-NLS-1$
		args['url'] = url; //$NON-NLS-1$
		this.updateProcessDescriptionWithURL(args);
	};
	
	/**
	 * @param {Object} args: url - the URL of the process description
	 */
	ProcessAuthoringClient.updateProcessDescriptionWithURL = function(args) {
		args.title = UIHelper.escapeXML(args.title);
		args.description = UIHelper.escapeXML(args.description);
		//process description in PLM, need to add share element
		var pdata = "<ProcessDescription xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" rdf:about=\"\" xml:lang=\"${language}\"> \r\n" + //$NON-NLS-1$
		"<dc:title>${title}</dc:title>\r\n" + //$NON-NLS-1$
		"<dc:description>${description}</dc:description>\r\n" + //$NON-NLS-1$
		"<default>${isDefault}</default>\r\n" +	//$NON-NLS-1$
		"<owner rdf:resource=\"${owner}\"/>\r\n" + //$NON-NLS-1$
		"<shared>${isPLMShared}</shared>\r\n";	//$NON-NLS-1$		
		
		pdata = dojo.string.substitute(pdata, args);
		var practicesNum = 0;
		
		dojo.forEach(args.practiceList, function(practice) {
			pdata += dojo.string.substitute("<practice shared=\"${0}\" dc:title=\"${1}\" rdf:resource=\"${2}\"/> \r\n", [practice.shared, UIHelper.escapeXML(practice.name), practice.url]); //$NON-NLS-1$
			practicesNum++;
		}, this);
				
		if (practicesNum === 0) {
			pdata += "<practice rdf:resource=\"\" />\r\n"; //$NON-NLS-1$	
		}
		
		pdata += "</ProcessDescription>"; //$NON-NLS-1$
		jazz.client.xhrPut({
			url: args.url + dojo.string.substitute("?updateType=partial&lang=${language}", args), //$NON-NLS-1$ //$NON-NLS-2$
			headers: {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"ETag": args.ETag //$NON-NLS-1$
			},
			handleAs: "text", //$NON-NLS-1$
			putData: pdata,
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.deleteAssociation = function(args) {
		jazz.client.xhrDelete({
			url: args.associationURL,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.getAssociation = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: args.url,
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.createAssociation = function(args) {
		var pdata = "<Association " + //$NON-NLS-1$
		"xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" " + //$NON-NLS-1$
		"xmlns:dc=\"http://purl.org/dc/terms/\" " + //$NON-NLS-1$
		"xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" " + //$NON-NLS-1$
		"rdf:about=\"\">" + //$NON-NLS-1$
		"<source rdf:resource=\"" + //$NON-NLS-1$
		args.sourceUrl +
		"\"/>" + //$NON-NLS-1$
		"<targets>" + //$NON-NLS-1$
		"  <target rdf:resource=\"" + //$NON-NLS-1$
		args.targetUrl +
		"\" dc:title=\"" + //$NON-NLS-1$
		UIHelper.escapeXML(args.targetName) +
		"\"/>" + //$NON-NLS-1$
		"</targets>" + //$NON-NLS-1$
		"</Association>"; //$NON-NLS-1$
		return jazz.client.xhrPost({
			url: ProcessAuthoringClient.getJtsProcessAuthoringRoot() + "associations/", //$NON-NLS-1$
			headers: {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
			},
			handleAs: "xml", //$NON-NLS-1$
			postData: pdata,
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.updateAssociation = function(args) {
		var newTargetsString = ""; //$NON-NLS-1$
		dojo.forEach(args.newTargets, function(target) {
			newTargetsString += "  <target rdf:resource=\"" + //$NON-NLS-1$
			target.url +
			"\" dc:title=\"" + //$NON-NLS-1$
			target.title +
			"\"/>"; //$NON-NLS-1$
		}, this);
		var pdata = "<Association " + //$NON-NLS-1$
		"xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" " + //$NON-NLS-1$
		"xmlns:dc=\"http://purl.org/dc/terms/\" " + //$NON-NLS-1$
		"xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" " + //$NON-NLS-1$
		"rdf:about=\"\">" + //$NON-NLS-1$
		"<source rdf:resource=\"" + //$NON-NLS-1$
		args.sourceUrl +
		"\"/>" + //$NON-NLS-1$
		"<targets>" + //$NON-NLS-1$
		newTargetsString +
		"</targets>" + //$NON-NLS-1$
		"</Association>"; //$NON-NLS-1$
		jazz.client.xhrPut({
			url: args.associationURL,
			headers: {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"ETag": args.ETag //$NON-NLS-1$
			},
			handleAs: "json", //$NON-NLS-1$
			putData: pdata,
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.searchPracticeByTitle = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: ProcessAuthoringClient.getJtsProcessAuthoringRoot() + "practices?searchString=" + encodeURIComponent(args.searchString), //$NON-NLS-1$
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.searchProcessDescriptionByTitle = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: ProcessAuthoringClient.getJtsProcessAuthoringRoot() + "descriptions?searchString=" + encodeURIComponent(args.searchString), //$NON-NLS-1$
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.getAllSharedPractices = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: JTS_PROCESS_AUTHORING_ROOT + "practices", //$NON-NLS-1$
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.getAllPLMProcessDescriptions = function(args) {
		args.requestQueryParam = "includePrivate=true"; //$NON-NLS-1$
		ProcessAuthoringClient._getAllProcessDescriptions(args);
	};
	
	ProcessAuthoringClient.getAllProcessDescriptions = function(args) {
		//default is shared process description
		if (!args.isPrivate) {
			args.isPrivate = false;
		}
		
		args.requestQueryParam = "private="+args.isPrivate; //$NON-NLS-1$
		ProcessAuthoringClient._getAllProcessDescriptions(args);
	};
	
	ProcessAuthoringClient._getAllProcessDescriptions = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: JTS_PROCESS_AUTHORING_ROOT + dojo.string.substitute("descriptions?${requestQueryParam}", args), //$NON-NLS-1$
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.getProcessDescription = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: args.resourceURL,
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.getProcessDescriptionOverview = function(args){
		var headers = {
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: args.resourceURL,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.getFrontingApplicaitonURIRoot = function(args) {
		return jazz.client.xhrGet({
			preventCache: true,
			url: args.resourceURL,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	ProcessAuthoringClient.createPracticeElementLanguage = function(args) {
	
		ProcessAuthoringClient.getPracticeElement({
			'url': args.resource + "?lang=" + args.language, //$NON-NLS-1$ //$NON-NLS-2$
			'load': function(data, ioArgs) { //$NON-NLS-1$
				var ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
				var pdata = "<PracticeElement xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xml:lang=\"" + args.language + "\"><presentation-type>description</presentation-type><dc:title>"; //$NON-NLS-1$ //$NON-NLS-2$
				pdata += args.title;
				pdata += "</dc:title><dc:description>"; //$NON-NLS-1$
				pdata += args.description;
				pdata += "</dc:description>"; //$NON-NLS-1$
				pdata += "</PracticeElement>"; //$NON-NLS-1$
				jazz.client.xhrPut({
					url: args.resource + "?updateType=partial&lang=" + args.language, //$NON-NLS-1$
					headers: {
						"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
						"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
						"Accept": "application/xml", //$NON-NLS-1$ //$NON-NLS-2$
						"ETag": ETag //$NON-NLS-1$
					},
					handleAs: "xml", //$NON-NLS-1$
					putData: pdata,
					error: args.error,
					load: args.load
				});
			}
		});
	};
	
	ProcessAuthoringClient.createProcessDescriptionLanguage = function(args) {
		ProcessAuthoringClient.getProcessDescription({
			'resourceURL': args.resource + "?lang=" + args.language, //$NON-NLS-1$ //$NON-NLS-2$
			'load': function(data, ioArgs) { //$NON-NLS-1$
				var practices = UIHelper.getElementsByTagNameNS(data, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_PRACTICE);
				var practiceXML = ""; //$NON-NLS-1$
				dojo.forEach(practices, function(practice) {
					var title = UIHelper.getNodeDcTitle(practice);
					var resource = UIHelper.getNodeResource(practice);
					practiceXML += dojo.string.substitute(PRACTICE_ITEM, {
						'title': title, //$NON-NLS-1$
						'resource': resource //$NON-NLS-1$
					});
				}, this);
				
				var pd = new PracticeElement({
					xml: data
				});
				var title = pd.title;
				var description = pd.description;
				var ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
				var postData = "<ProcessDescription xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" rdf:about=\"\" xml:lang=\"" + args.language + "\"> \r\n" + //$NON-NLS-1$ //$NON-NLS-2$
				"<dc:title>" + //$NON-NLS-1$
					title +
				"</dc:title>" + //$NON-NLS-1$ 
				"<dc:description>" + //$NON-NLS-1$
					description +
				"</dc:description>" + //$NON-NLS-1$ 
				"<owner rdf:resource=\"" + //$NON-NLS-1$ 
					args.owner +
				"\"/>\r\n" + //$NON-NLS-1$ 
				practiceXML +
				"</ProcessDescription>"; //$NON-NLS-1$
				jazz.client.xhrPut({
					url: args.resource + "?updateType=partial&lang=" + args.language, //$NON-NLS-1$
					headers: {
						"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
						"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
						"ETag": ETag //$NON-NLS-1$
					},
					handleAs: "text", //$NON-NLS-1$
					putData: postData,
					error: args.error,
					load: args.load
				});
			},
			'error': args.error //$NON-NLS-1$
		});
	};
	
	ProcessAuthoringClient.createPracticeLanguage = function(args) {
		ProcessAuthoringClient.getPracticeElement({
			'url': args.resource + "?lang=" + args.language, //$NON-NLS-1$ //$NON-NLS-2$
			'load': function(data, ioArgs) { //$NON-NLS-1$
				var ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
				var pdata = "<Practice " + //$NON-NLS-1$
				"xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" " + //$NON-NLS-1$
				"xmlns:dc=\"http://purl.org/dc/terms/\" " + //$NON-NLS-1$
				"xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" " + //$NON-NLS-1$
				"rdf:about=\"\" " + //$NON-NLS-1$
				"xml:lang=\"" + //$NON-NLS-1$
				args.language +
				"\"> " + //$NON-NLS-1$ //$NON-NLS-2$
				"<presentation-type>structure</presentation-type>" + //$NON-NLS-1$
				"<dc:title>" + //$NON-NLS-1$
				args.title +
				"</dc:title>" + //$NON-NLS-1$  //$NON-NLS-2$
				"<dc:description>" + //$NON-NLS-1$
				args.description +
				"</dc:description>" + //$NON-NLS-1$ //$NON-NLS-2$
				"</Practice>"; //$NON-NLS-1$
				jazz.client.xhrPut({
					url: args.resource + "?updateType=partial&lang=" + args.language, //$NON-NLS-1$
					headers: {
						"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
						"Content-Type": "application/xml;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
						"Accept": "application/xml", //$NON-NLS-1$ //$NON-NLS-2$
						"ETag": ETag //$NON-NLS-1$
					},
					handleAs: "xml", //$NON-NLS-1$
					putData: pdata,
					error: args.error,
					load: args.load
				});
			},
			'error': args.error //$NON-NLS-1$
		});
	};
	
	ProcessAuthoringClient.getAttachmentsByPractice = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: args.url + "/attachments/", //$NON-NLS-1$
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.getAttachment = function(args) {
		var headers = {
			"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (args.lastModified) {
			headers[ProcessAuthoringConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		jazz.client.xhrGet({
			preventCache: true,
			url: args.url, //$NON-NLS-1$
			headers: headers,
			handleAs: "xml", //$NON-NLS-1$
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.deleteAttachment = function(args) {
		jazz.client.xhrDelete({
			preventCache: true,
			url: args.url, //$NON-NLS-1$
			headers: {
				Accept: "application/xml" //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			error: args.error,
			load: args.load
		});
	};
	
	ProcessAuthoringClient.getJtsUriRoot = function(args) {
		return JTS_URI_ROOT;
	}
	
	ProcessAuthoringClient.getJtsProcessAuthoringRoot = function(args) {
		return JTS_PROCESS_AUTHORING_ROOT;
	}
	
	function getJtsRoot() {
		var discoverySvcUrl = net.jazz.ajax.BootstrapProperties.DISCOVERYSERVICE_URL;
		var index = discoverySvcUrl.lastIndexOf('/'); //$NON-NLS-1$
		if(index == discoverySvcUrl.length - 1) {
			index = discoverySvcUrl.lastIndexOf('/', index); //$NON-NLS-1$
		}
		return discoverySvcUrl.substring(0, index);
	}

})();
