/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.internal.UIHelper");//$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$ 
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("dojox.xmpp.util"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.BootstrapProperties"); //$NON-NLS-1$
	
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants;
	var UIHelper = com.ibm.team.process.authoring.web.common.internal.UIHelper;
	var ns = [];
	ns[ProcessAuthoringConstants.NAMESPACE_RDF] = "rdf:"; //$NON-NLS-1$
	ns[ProcessAuthoringConstants.NAMESPACE_DC] = "dc:"; //$NON-NLS-1$
	ns[ProcessAuthoringConstants.NAMESPACE_JTP] = "jtp:"; //$NON-NLS-1$
	
	function isNode(o) {
		return o && o.nodeType && o.cloneNode;
	}

    var getRegForTag = function(tag, type) {
    	tag = tag || ''; //$NON-NLS-1$
		var regstr = [  "<" + tag + "[^>]*>([\\s\\S]*?)<\/" + tag + ">", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						"<\/?" + tag + "([^>]+)?>", //$NON-NLS-1$ //$NON-NLS-2$
						"<" + tag + "[^>]*>|<\/"+tag+">" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					];
    	return new RegExp(regstr[type], "gi"); //$NON-NLS-1$
    };
    
    var stripContentTagArray = []; //$NON-NLS-1$
    var singleTagArray = ['br', 'hr']; //$NON-NLS-1$ //$NON-NLS-2$
	var stripTagArray = ['table', 'span', 'tr', 'td', 'tbody', 'th', 'img', 'p', 'b', 'a', 'i','ol','li','ul', 'blockquote','div','sup','sub','u','strike']; //$NON-NLS-10$ //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-11$ //$NON-NLS-12$ //$NON-NLS-13$ //$NON-NLS-14$ //$NON-NLS-15$ //$NON-NLS-16$ //$NON-NLS-17$ //$NON-NLS-18$ //$NON-NLS-19$ //$NON-NLS-20$
	
	com.ibm.team.process.authoring.web.common.internal.UIHelper = {
		
		/*
		 * Retrieves the width of the longest string in the array of
		 * strings passed to it. Can be used to determine the correct size for buttons.
		 * Returns the width in EM.
		 */
		getMaxStringDisplayWidthInEms: function(stringArray, testNode, pxMinWidth){
			if (!isNode(testNode)) {
				return -1;
			}
			var tester = document.createElement("div"); //$NON-NLS-1$
			dojo.style(tester, "position", "absolute"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "left", "-10000px"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "pad", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(tester, "border", "0"); //$NON-NLS-1$ //$NON-NLS-2$
			testNode.appendChild(tester);
			var width = pxMinWidth || -1;
			
			// Find the longest string
			dojo.forEach(stringArray, function(sample){
				var sampleDiv = document.createElement("div"); //$NON-NLS-1$
				dojo.style(sampleDiv, "float", "right"); //$NON-NLS-1$ //$NON-NLS-2$
				sampleDiv.innerHTML = sample;
				tester.appendChild(sampleDiv);
				width = sampleDiv.clientWidth > width ? sampleDiv.clientWidth : width;
				tester.removeChild(sampleDiv);
				sampleDiv = null;
			});
			
			// Compute pixels per EM
			tester.style.height = '100em'; //$NON-NLS-1$
			var emSize = tester.offsetHeight / 100;
			
			testNode.removeChild(tester);
			
			return width / emSize;
		},
		
		/*
		 *  Helper to style the button for I18N support.  Adds the desired pad
		 *  to the width and sets the button width in em.  Removes any left
		 *  or right pad on the button to ensure that it doesn't eat into the
		 *  content space
		 */
		styleI18NButton: function(button, emWidth, emPad){
			if (emPad === undefined) {
				emPad = 1; //  Set a default pad
			}
			var width = emWidth + emPad + "em"; //$NON-NLS-1$
			dojo.style(button, "width", width); //$NON-NLS-1$
			dojo.style(button, "paddingLeft", 0); //$NON-NLS-1$
			dojo.style(button, "paddingRight", 0); //$NON-NLS-1$
		},
		
		/*
		 * Resize the buttons in the array to accomodate the width of their labels.
		 *
		 * testNode - The node that can be used to calculate the required button width
		 * buttons - The array of buttons with their labels set
		 * pxMinWidth - The optional min width of the buttons in pixels
		 * emPad - The option left and right pad to give the button on each side of label
		 */
		resizeI18NButtons: function(testNode, buttons, pxMinWidth, emPad){
			var labels = [];
			for (var i = 0; i < buttons.length; i++) {
				labels[i] = buttons[i].innerHTML;
			}
			var newWidth = this.getMaxStringDisplayWidthInEms(labels, testNode, pxMinWidth);
			dojo.forEach(buttons, function(button){
				this.styleI18NButton(button, newWidth, emPad);
			}, this);
			return newWidth;
		},
		
		
		isSpecifiedNode: function(node, baseName, namespaceURI) {
			if ((node.baseName == baseName || node.localName == baseName) && (node.namespaceURI == namespaceURI || (node.namespaceURI === "" && namespaceURI === null))) { //$NON-NLS-1$
				return true;
			}
			return false;
		},
		
		getNode: function(collection, baseName, namespaceURI) {
			var matchingNode;
			dojo.some(collection, function(node) {
				if (this.isSpecifiedNode(node, baseName, namespaceURI)) {
					matchingNode = node;
					return true;
				}
				return false;
			}, this);
			return matchingNode;
		},
		
		getNodes: function(collection, baseName, namespaceURI) {
			var matchingNodes = [];
			dojo.forEach(collection, function(node) {
				if (this.isSpecifiedNode(node, baseName, namespaceURI)) {
					matchingNodes.push(node);
				}
			}, this);
			return matchingNodes;
		},
		
		getAttributeNode: function(node, name, namespace) {
			var attribute;
			if (dojo.isIE) {
				attribute = node.attributes.getQualifiedItem(name, namespace);
			} else {
				attribute = node.attributes.getNamedItemNS(namespace, name);
			}
			
			//for bug 117488, can't set attribute for a node which namespace is different from the attribute
			if ( !attribute && ns[namespace] ) {
				attribute = node.getAttributeNode(ns[namespace]+name);
			}
			return attribute;
		},
		
		getPracticeElementPresentation: function(xml) {
			var presentation = this.getNode(xml.documentElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRESENTATION, ProcessAuthoringConstants.NAMESPACE_JTP);
			return presentation;
		},
		
		getSections: function(practiceElement) {
			var presentation = this.getNode(practiceElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRESENTATION, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (presentation) {
				return this.getNodes(presentation.childNodes, ProcessAuthoringConstants.XMLELEMENT_SECTION, ProcessAuthoringConstants.NAMESPACE_JTP);
			}
			return null;
		},
		
		getSectionLabel: function(section) {
			var labelNode = this.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
			if (labelNode) {
				return this.unescapeXML(labelNode.value);
			}
			return ""; //$NON-NLS-1$
		},
		
		isNewCreated: function(node) {
			var newCreatedNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_NEW_CREATED, ""); //$NON-NLS-1$
			if (newCreatedNode) {
				return newCreatedNode.value === ProcessAuthoringConstants.VALUE_TRUE;
			}
			return false; //$NON-NLS-1$
		},
		
		getFieldType: function(field) {
			var typeNode = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_TYPE, ""); //$NON-NLS-1$
			if (typeNode) {
				return typeNode.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		getFieldLabel: function(field) {
			var labelNode = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
			if (labelNode) {
				return this.unescapeXML(labelNode.value);
			}
			return ""; //$NON-NLS-1$
		},
		
		getFieldName: function(field) {
			var nameNode = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_NAME, ""); //$NON-NLS-1$
			if (nameNode) {
				return nameNode.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		getFieldText: function(field) {
			if (field.textContent) {
				return field.textContent;
			} else if (field.firstChild) {
				return this.unescapeXML(field.firstChild.data);
			} else {
				return ''; //$NON-NLS-1$
			}
		},
		
		setNodeText: function(node, value) {
			if (typeof node.textContent !== "undefined") { //$NON-NLS-1$
				node.textContent = value;
			} else if (node.firstChild) {
				node.firstChild.data = value;
			}
		},
		
		getPracticeElementTitle: function(practiceElement) {
			var title = this.getNode(practiceElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_TITLE, ProcessAuthoringConstants.NAMESPACE_DC);
			if (title.firstChild) {
				return this.unescapeXML(title.firstChild.data);
			}
			return "&lt;No Title&gt;"; //$NON-NLS-1$
		},
		
		getLanguages: function(element) {
			var languages = this.getNode(element.childNodes, ProcessAuthoringConstants.XMLELEMENT_LANGUAGES, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (languages && languages.firstChild) {
				return languages.firstChild.data;
			}
			return null; //$NON-NLS-1$
		},
		
		getPracticeElementDescription: function(practiceElement) {
			var description = this.getNode(practiceElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION, ProcessAuthoringConstants.NAMESPACE_DC);
			if (description && description.firstChild) {
				return this.unescapeXML(description.firstChild.data);
			}
			return ProcessAuthoringConstants.EMPTY; //$NON-NLS-1$
		},
		
		getFields: function(section) {
			return this.getNodes(section.childNodes, ProcessAuthoringConstants.XMLELEMENT_FIELD, ProcessAuthoringConstants.NAMESPACE_JTP);
		},
		
		getProcessDescriptionValues: function(processDescriptionNode) {
			var aboutAttribute = this.getAttributeNode(processDescriptionNode, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF);
			var nameNode = this.getNode(processDescriptionNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_TITLE, ProcessAuthoringConstants.NAMESPACE_DC);
			var descriptionNode = this.getNode(processDescriptionNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION, ProcessAuthoringConstants.NAMESPACE_DC); //$NON-NLS-1$			
			return {
				name: nameNode && nameNode.firstChild ? this.unescapeXML(nameNode.firstChild.data) : ProcessAuthoringConstants.EMPTY,
				url: aboutAttribute ? aboutAttribute.value : ProcessAuthoringConstants.EMPTY,
				description: descriptionNode && descriptionNode.firstChild ? this.unescapeXML(descriptionNode.firstChild.data) : ProcessAuthoringConstants.EMPTY
			};
		},
		
		getProcessDescriptionTitle: function(xml) {
			var processDescription = this.getNode([xml.documentElement], "ProcessDescription", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			var nameNode = this.getNode(processDescription.childNodes, "title", ProcessAuthoringConstants.NAMESPACE_DC); //$NON-NLS-1$ 
			if (nameNode) {
				var name = this.unescapeXML(nameNode.firstChild.data);
				return name;
			} else {
				return ""; //$NON-NLS-1$
			}
			
		},
		
		getProcessDescriptionDescription: function(xml) {
			var processDescription = this.getNode([xml.documentElement], "ProcessDescription", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			var descriptionNode = this.getNode(processDescription.childNodes, "description", ProcessAuthoringConstants.NAMESPACE_DC); //$NON-NLS-1$
			var description = ""; //$NON-NLS-1$
			if (descriptionNode && descriptionNode.firstChild) {
				description = descriptionNode.firstChild.data;
			}
			return this.unescapeXML(description);
		},
		
		getProcessDescriptionOwnerURL: function(xml) {
			var processDescription = this.getNode([xml.documentElement], "ProcessDescription", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			var xmlOwnerNode = this.getNode(processDescription.childNodes, "owner", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			if (xmlOwnerNode) {
				var ownerAttribute = this.getAttributeNode(xmlOwnerNode, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
				var ownerURI = ownerAttribute.value;
			}
			return ownerURI;
		},
		
		getProcessDescriptionOwnerName: function(xml) {
			var processDescription = this.getNode([xml.documentElement], "ProcessDescription", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			var xmlOwnerNode = this.getNode(processDescription.childNodes, "owner", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			if (xmlOwnerNode) {
				var attribute = this.getAttributeNode(xmlOwnerNode, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
				var ownerName = attribute.value;
			}
			return ownerName;
		},
		
		getLanguage: function(node) {
			var languageNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_LANGUAGE, ProcessAuthoringConstants.NAMESPACE_XML); //$NON-NLS-1$
			if (languageNode) {
				return languageNode.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		isSharedElement: function(xml) {
			var root = xml.documentElement;
			var sharedNode = this.getNode(root.childNodes, ProcessAuthoringConstants.XMLELEMENT_SHARED, ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			if (sharedNode && sharedNode.firstChild) {
				var value = sharedNode.firstChild.data;
				if (value == "true") { //$NON-NLS-1$
					return true;
				}
			}
			return false;
		},
		
		getPracticeElement: function(xml) {
			return this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_PRACTICEELEMENT_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
		},
		
		getPracticeElementOwner: function(xml) {
			var rdf = this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF);
			var xmlOwnerNode = this.getNode(rdf.childNodes, "owner", null); //$NON-NLS-1$
			if (xmlOwnerNode) {
				var ownerAttribute = this.getAttributeNode(xmlOwnerNode, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
				var ownerURI = ownerAttribute.value;
			}
			return ownerURI;
		},
		
		getPresentation: function(xml) {
			var presentation = this.getNode(xml.documentElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRESENTATION, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (!presentation) {
				presentation = this.getEmptyPresentation();
			}
			return presentation;
		},
		
		getEmptyPresentation: function() {
			var xml = "<Practice xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\"><presentation rdf:parseType=\"Literal\"></presentation></Practice>"; //$NON-NLS-1$
			var doc = dojox.xml.parser.parse(xml);
			return this.getNode(doc.documentElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRESENTATION, ProcessAuthoringConstants.NAMESPACE_JTP);
		},
		
		getPracticePresentation: function(xml) {
			var practice = this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_PRACTICE_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (practice) {
				return this.getNode(practice.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRESENTATION, ProcessAuthoringConstants.NAMESPACE_JTP);
			}
			return null;
		},
		
		getTitle: function(xml) {
			return this.unescapeXML(this.getNode(xml.documentElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_TITLE, ProcessAuthoringConstants.NAMESPACE_DC).firstChild.data);			
		},
		
		getPracticeTitle: function(xml) {
			var practice = this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_PRACTICE_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (practice) {
				return this.unescapeXML(this.getNode(practice.childNodes, ProcessAuthoringConstants.XMLELEMENT_TITLE, ProcessAuthoringConstants.NAMESPACE_DC).firstChild.data);
			}
			return ""; //$NON-NLS-1$
		},
		
		getDescription: function(xml) {
			var description = this.getNode(xml.documentElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION, ProcessAuthoringConstants.NAMESPACE_DC);
			return (description && description.firstChild) ? this.unescapeXML(description.firstChild.data) : ""; //$NON-NLS-1$
		},
		
		getPracticeDescription: function(xml) {
			var practice = this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_PRACTICE_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (practice) {
				var description = this.getNode(practice.childNodes, ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION, ProcessAuthoringConstants.NAMESPACE_DC);
				return (description && description.firstChild) ? this.unescapeXML(description.firstChild.data) : ""; //$NON-NLS-1$
			}
			return ""; //$NON-NLS-1$
		},
		
		getResourceUrl: function(node) {
			var aboutNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF); //$NON-NLS-1$
			if (aboutNode) {
				return aboutNode.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		getPracticeUrl: function(xml) {
			var practice = this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_PRACTICE_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (practice) {
				return this.getAttributeNode(practice, ProcessAuthoringConstants.XMLATTRIBUTE_ABOUT, ProcessAuthoringConstants.NAMESPACE_RDF).value;
			}
			return ""; //$NON-NLS-1$
		},
		
		addPracticeElement: function(practiceNode, practiceElement, language, parentId) {
			var presentation = practiceNode.model.structure;
			var parent = null;
			if (parentId) {
				parent = this.getGroupingNode(presentation, parentId)
			} else {
				parent = presentation;
			}
			var element = this.createElementNS(ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_ELEMENT);
			this.createAttributeNS(element, "", ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, practiceElement.label); //$NON-NLS-1$
			this.createAttributeNS(element, ProcessAuthoringConstants.NAMESPACE_RDF, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, practiceElement.id);
			parent.appendChild(element);
			
			practiceNode.languages[language].textContent[practiceElement.id] = {label: practiceElement.label};
		},
		
		updatePracticeElement: function(practiceNode, language, id, label) {
			practiceNode.languages[language].textContent[id] = {label: label};
		},
		
		deletePracticeElement: function(practiceNode, id) {
			var presentation = practiceNode.model.structure;
			var elementNode = this.getPracticeElementNode(presentation, id);
			elementNode.parentNode.removeChild(elementNode);
		},
		
		updatePracticeElementResource: function(practiceNode, oldResource, newResource) {
			var presentation = practiceNode.model.structure;
			var elementNode = this.getPracticeElementNode(presentation, oldResource);
			var resourceAttr = this.getAttributeNode(elementNode, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
			resourceAttr.value = newResource;
		},
		
		addCategory: function(practiceNode, categoryNode, language, parentId) {
			var presentation = practiceNode.model.structure;
			var parent = null;
			if (parentId) {
				parent = this.getGroupingNode(presentation, parentId)
			} else {
				parent = presentation;
			}
			var grouping = this.createElementNS(ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_GROUPING);
			this.createAttributeNS(grouping, "", ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, categoryNode.label); //$NON-NLS-1$
			this.createAttributeNS(grouping, ProcessAuthoringConstants.NAMESPACE_RDF, ProcessAuthoringConstants.XMLATTRIBUTE_ID, categoryNode.id);
			this.createAttributeNS(grouping, "", ProcessAuthoringConstants.XMLATTRIBUTE_NEW_CREATED, "true"); //$NON-NLS-1$ //$NON-NLS-2$
			parent.appendChild(grouping);
			
			practiceNode.languages[language].textContent[categoryNode.id] = {label: categoryNode.label};
		},
		
		updateCategory: function(practiceNode, language, id, label) {
			practiceNode.languages[language].textContent[id] = {label: label};
		},
		
		deleteCategory: function(practiceNode, id) {
			var presentation = practiceNode.model.structure;
			var categoryNode = this.getGroupingNode(presentation, id);
			categoryNode.parentNode.removeChild(categoryNode);
		},
		
		moveNode: function(structure, node, targetParent, position) {
			if (node.type === ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
				node = this.getGroupingNode(structure, node.id);
			} else {
				if (node.status === ProcessAuthoringConstants.STATUS_NEW) {
					node = this.getPracticeElementNode(structure, node.id);
				} else {
					node = this.getPracticeElementNode(structure, node.resource);
				}
			}
			if (targetParent.type === ProcessAuthoringConstants.NAVIGATORTYPE_CATEGORY) {
				targetParent = this.getGroupingNode(structure, targetParent.id);
			} else {
				targetParent = structure;
			}
			
			if (position !== -1) {
				node.parentNode.removeChild(node);
				var cn = targetParent.childNodes;
				var count = 0;
				var pos = 0;
				for (var i = 0; i < cn.length; i++) {
					if (count == position) {
						pos = i;
						break;
					}
					
					if (cn[i].nodeType === 1) {
						count++;	
					}
				}
				
				if (count < position) {
					targetParent.appendChild(node);
				} else {
					dojo.place(node, targetParent, pos);
				}
			}
		},
		
		isPractice: function(node) {
			return this.isSpecifiedNode(node, ProcessAuthoringConstants.XMLELEMENT_PRACTICE, ProcessAuthoringConstants.NAMESPACE_JTP);
		},
		
		isPracticeElement: function(node) {
			return this.isSpecifiedNode(node, ProcessAuthoringConstants.XMLELEMENT_ELEMENT, ProcessAuthoringConstants.NAMESPACE_JTP);
		},
		
		isCategoryNode: function(node) {
			return this.isSpecifiedNode(node, ProcessAuthoringConstants.XMLELEMENT_GROUPING, ProcessAuthoringConstants.NAMESPACE_JTP);
		},
		
		getNodeLabel: function(node) {
			var labelNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
			if (labelNode) {
				return this.unescapeXML(labelNode.value);
			}
			return ""; //$NON-NLS-1$
		},
		
		getNodeTitle: function(node) {
			return this.unescapeXML(this.getNode(node.childNodes, ProcessAuthoringConstants.XMLATTRIBUTE_TITLE, ProcessAuthoringConstants.NAMESPACE_DC).firstChild.data);
		},
		
		getNodeId: function(node) {
			var idNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
			if (idNode) {
				return idNode.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		getNodeDcTitle: function(node) {
			var labelNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_TITLE, ProcessAuthoringConstants.NAMESPACE_DC); //$NON-NLS-1$
			if (labelNode) {
				return this.unescapeXML(labelNode.value);
			}
			return ""; //$NON-NLS-1$
		},
		
		getNodeShareAttr: function(node) {
			var SharedNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_SHARED, ProcessAuthoringConstants.NAMESPACE_DC); 
			if (SharedNode) {
				return this.unescapeXML(SharedNode.value).toLowerCase() === "true" ? true:false; //$NON-NLS-1$
			}
			return false; 
		},

		getNodeResource: function(node) {
			var resourceNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
			if (resourceNode) {
				return resourceNode.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		getNodeIcon: function(node) {
			var iconNode = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_ICON, ""); //$NON-NLS-1$
			if (iconNode) {
				return iconNode.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		getAllPracticesFromProcessDescription: function(xml) {
			var practiceInfo = [];
			var urlMap = {};
			var processDescription = this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_PROCESSDESCRIPTION_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (processDescription) {
				var practices = this.getNodes(processDescription.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRACTICE, ProcessAuthoringConstants.NAMESPACE_JTP);
				dojo.forEach(practices, function(node) {
					var attrResource = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
					var attrTitle = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_TITLE, ProcessAuthoringConstants.NAMESPACE_DC);
					var attrShared = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_SHARED, ""); //$NON-NLS-1$
					if (attrResource && attrTitle) { //resource and title are required attribute					
						if (!urlMap[attrResource.value]) {
							practiceInfo.push({
								url: attrResource.value,
								title: this.unescapeXML(attrTitle.value),
								shared: attrShared ? attrShared.value : "false" //$NON-NLS-1$
							});
							urlMap[attrResource.value] = true;
						}
					}
				}, this);
			}
			return practiceInfo;
		},
		
		getLinkSource: function(linkNode) {
			if (linkNode) {
				var source = this.getNode(linkNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_SOURCE, ProcessAuthoringConstants.NAMESPACE_JTP);
				if (source) {
					return this.getNodeResource(source);
				}
			}
			return ""; //$NON-NLS-1$
		},
		
		getLinkTarget: function(linkNode) {
			if (linkNode) {
				var source = this.getNode(linkNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_TARGET, ProcessAuthoringConstants.NAMESPACE_JTP);
				if (source) {
					return this.getNodeResource(source);
				}
			}
			return ""; //$NON-NLS-1$
		},
		
		getLinks: function(xml) {
			var links = [];
			var rdfNode = this.getNode([xml.documentElement], ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF);
			if (rdfNode) {
				var linkNodes = this.getNodes(rdfNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_LINK, ProcessAuthoringConstants.NAMESPACE_JTP);
				dojo.forEach(linkNodes, function(linkNode) {
					var link = {};
					link.source = this.getLinkSource(linkNode);
					link.target = this.getLinkTarget(linkNode);
					link.name = this.getLinkName(linkNode);
					links.push(link);
				}, this);
			}
			return links;
		},
		
		getLinkName: function(linkNode) {
			var titleNode = this.getNode(linkNode.childNodes, "targetTitle", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			return titleNode.firstChild.data;
		},
		
		getGroupingNode: function(presentation, id) {
			var groupingNode = null;
			var groupings = this.getElementsByTagNameNS(presentation, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_GROUPING);
			
			dojo.some(groupings, function(node) {
				var groupingId = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
				if ((groupingId) && (groupingId.value == id)) {
					groupingNode = node;
					return true;
				}
				return false;
			}, this);
			return groupingNode;
		},
		
		getPracticeElementNode: function(presentation, resource) {
			var elementNode = null;
			var elements = this.getElementsByTagNameNS(presentation, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_ELEMENT);
			
			dojo.some(elements, function(node) {
				var elementResource = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
				if ((elementResource) && (elementResource.value == resource)) {
					elementNode = node;
					return true;
				}
				return false;
			}, this);
			return elementNode;
		},
		
		getElementsByTagNameNS: function(node, namespace, tagName) {
			if (node.getElementsByTagNameNS) {
				return node.getElementsByTagNameNS(namespace, tagName);
			} else {
				var nodes = node.getElementsByTagName(tagName);
				
				return this.getNodes(nodes, tagName, namespace);
			}
		},
		
		getAttachmentNodes: function(xml) {
			return this.getNodes(xml.documentElement.childNodes, ProcessAuthoringConstants.XMLELEMENT_ATTACHMENT, ProcessAuthoringConstants.NAMESPACE_JTP);
		},
		
		getAttachmentPath: function(attachment) {
			var node = this.getNode(attachment.childNodes, ProcessAuthoringConstants.XMLELEMENT_PATH, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (node) {
				return node.firstChild.data;
			}
			return ""; //$NON-NLS-1$
		},
		
		getAttachmentSize: function(attachment) {
			var node = this.getNode(attachment.childNodes, ProcessAuthoringConstants.XMLELEMENT_SIZE, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (node) {
				return node.firstChild.data;
			}
			return ""; //$NON-NLS-1$
		},
		
		getAttachmentService: function(attachment) {
			var node = this.getNode(attachment.childNodes, ProcessAuthoringConstants.XMLELEMENT_SERVICE, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (node && node.firstChild) {
				return node.firstChild.data;
			}
			return ""; //$NON-NLS-1$
		},
		
		getAttachmentDate: function(attachment) {
			var node = this.getNode(attachment.childNodes, ProcessAuthoringConstants.XMLELEMENT_CREATED_DATE, ProcessAuthoringConstants.NAMESPACE_DC);
			if (node) {
				return node.firstChild.data;
			}
			return ""; //$NON-NLS-1$
		},
		
		getAttachmentCreator: function(attachment) {
			var node = this.getNode(attachment.childNodes, ProcessAuthoringConstants.XMLELEMENT_CREATOR, ProcessAuthoringConstants.NAMESPACE_DC);
			if (node) {
				var attribute = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
				return attribute.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		getAttachmentContent: function(attachment) {
			var node = this.getNode(attachment.childNodes, ProcessAuthoringConstants.XMLELEMENT_CONTENT, ProcessAuthoringConstants.NAMESPACE_JTP);
			if (node) {
				var attribute = this.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
				return attribute.value;
			}
			return ""; //$NON-NLS-1$
		},
		
		
		getPracticeElementXML: function(practiceElement, language, isPractice) {
			var title = practiceElement.languages[language].basicInformation.label;
			var description = practiceElement.languages[language].basicInformation.description;
			var presentation = this.updatePracticeElementPresentation(practiceElement.model, practiceElement.languages[language].textContent);
			return dojox.xml.parser.parse(this.buildPracticeElementXML(title, description, presentation, language, isPractice));
		},
		
		buildPracticeElementXML: function(title, description, presentation, language) {
			if ( presentation ) {
				presentation = dojox.xml.parser.innerXML(presentation);	
			} else {
				presentation = "<presentation/>" //$NON-NLS-1$
			}
			
			var pdata = "<PracticeElement xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xml:lang=\"" + language + "\"><presentation-type>description</presentation-type><dc:title>"; //$NON-NLS-1$ //$NON-NLS-2$
				pdata += this.escapeXML(title);
				pdata += "</dc:title><dc:description>"; //$NON-NLS-1$
				pdata += this.escapeXML(description);
				pdata += "</dc:description>"; //$NON-NLS-1$
				pdata += presentation; //$NON-NLS-1$
				pdata += "</PracticeElement>"; //$NON-NLS-1$

			return pdata;
		},

		buildPracticeXML: function(title, description, presentation, language, presentationType, isDefault) {
			if ( presentation ) {
				presentation = dojox.xml.parser.innerXML(presentation);	
			} else {
				presentation = "<presentation/>" //$NON-NLS-1$
			}
			
			if ( !presentationType || typeof(presentationType) === "undefined" ) { //$NON-NLS-1$
				presentationType = "description"; //$NON-NLS-1$
			}
			
			var pdata = "<Practice xmlns=\"http://jazz.net/xmlns/prod/jazz/jtp/0.6/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dc=\"http://purl.org/dc/terms/\" xml:lang=\"" + language + "\"><presentation-type>${0}</presentation-type><dc:title>"; //$NON-NLS-1$ //$NON-NLS-2$
				pdata += this.escapeXML(title);
				pdata += "</dc:title><dc:description>"; //$NON-NLS-1$
				pdata += description;
				pdata += "</dc:description>"; //$NON-NLS-1$
			if(typeof(isDefault) !== "undefined") { //$NON-NLS-1$
				pdata += "<default>" + isDefault + "</default>"; //$NON-NLS-1$ //$NON-NLS-2$
			}
				pdata += presentation; //$NON-NLS-1$
				pdata += "</Practice>"; //$NON-NLS-1$

			return dojo.string.substitute(pdata, [presentationType]);
		},

		
		updateNewElements: function(practiceElement, newElements, newSections, newFields) {
			var updatedTextContent = {};
			var i;
			if (newElements.newSections && newSections.length === newElements.newSections.length ) {
				for (i = 0; i < newElements.newSections.length; i++) {
					this.createAttributeNS(newSections[i].node, ProcessAuthoringConstants.NAMESPACE_RDF, ProcessAuthoringConstants.XMLATTRIBUTE_ID, newElements.newSections[i]);
					updatedTextContent[newSections[i].id] = newElements.newSections[i];
				}
			}
			
			if (newElements.newFields && newFields.length === newElements.newFields.length ) {
				for (i = 0; i < newElements.newFields.length; i++) {
					this.createAttributeNS(newFields[i].node, ProcessAuthoringConstants.NAMESPACE_RDF, ProcessAuthoringConstants.XMLATTRIBUTE_ID, newElements.newFields[i]);
					updatedTextContent[newFields[i].id] = newElements.newFields[i];
				}
			}
			
			for (var language in practiceElement.languages) {
				if (practiceElement.languages[language].status === ProcessAuthoringConstants.STATUS_UPDATE) {
					for (var key in updatedTextContent) {
						var textContent = practiceElement.languages[language].textContent;
						if (textContent[key]) {
							textContent[updatedTextContent[key]] = textContent[key];
						}
					}
				}
			}
		},
		
		updateNewGroups: function(practice, newElements, newGroups, updateCurrentItem) {
			var updatedTextContent = {};
			var i;
			if (newElements.newGroupings && newGroups.length === newElements.newGroupings.length ) {
				for (i = 0; i < newElements.newGroupings.length; i++) {
					this.createAttributeNS(newGroups[i].node, ProcessAuthoringConstants.NAMESPACE_RDF, ProcessAuthoringConstants.XMLATTRIBUTE_ID, newElements.newGroupings[i]);
					updatedTextContent[newGroups[i].id] = newElements.newGroupings[i];
					updateCurrentItem(newGroups[i].id, newElements.newGroupings[i]);
				}
			}
			
			for (var language in practice.languages) {
				if (practice.languages[language].status === ProcessAuthoringConstants.STATUS_UPDATE) {
					for (var key in updatedTextContent) {
						var textContent = practice.languages[language].textContent;
						if (textContent[key]) {
							textContent[updatedTextContent[key]] = textContent[key];
						}
					}
				}
			}
		},
		
		updatePracticeElementModel: function(practiceElement, newSections, newFields) {
			var model = practiceElement.model.data;
			if ( !model || typeof model === "undefined") { //$NON-NLS-1$
				return;
			}
			var sections = this.getAllSections(model);
            dojo.forEach(sections, function(section) {
            	var newCreatedAttribute = this.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_NEW_CREATED, ""); //$NON-NLS-1$
            	if (newCreatedAttribute && newCreatedAttribute.value == ProcessAuthoringConstants.VALUE_TRUE) {
            		var idAttribute = this.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            		var id = idAttribute.value;
            		section.removeAttributeNode(newCreatedAttribute);
            		section.removeAttributeNode(idAttribute);
            		newSections.push({
            			'node': section, //$NON-NLS-1$
            			'id': id //$NON-NLS-1$
            		});
            	}
            }, this);
			
			var fields = this.getAllFields(model);
            dojo.forEach(fields, function(field) {
            	var newCreatedAttribute = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_NEW_CREATED, ""); //$NON-NLS-1$
            	if (newCreatedAttribute && newCreatedAttribute.value == ProcessAuthoringConstants.VALUE_TRUE) {
            		var idAttribute = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            		var id = idAttribute.value;
            		field.removeAttributeNode(newCreatedAttribute);
            		field.removeAttributeNode(idAttribute);
            		newFields.push({
            			'node': field, //$NON-NLS-1$
            			'id': id //$NON-NLS-1$
            		});
            	}
            }, this);
		},
		
		updatePracticeElementPresentation: function(model, textContent) {
			if ( !model.data || typeof model.data === "undefined") { //$NON-NLS-1$
				return null;
			}
			var sections = this.getAllSections(model.data);
            dojo.forEach(sections, function(section) {
            	var idAttr = this.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            	if (idAttr) {
            		var key = idAttr.value;
            		if (textContent[key]) {
            			var label = this.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
            			label.value = textContent[key].label;
            		}
            	}
            }, this);
			var fields = this.getAllFields(model.data);
			dojo.forEach(fields, function(field) {
            	var idAttr = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            	if (idAttr) {
					var key = idAttr.value;
					if (textContent[key]) {
						var label = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
						label.value = textContent[key].label;
						this.setNodeText(field, textContent[key].value);
					}
				}
			}, this);
			return model.data;
		},
		
		updatePracticeStructurePresentation: function(model, textContent) {
			if ( !model.structure || typeof model.structure === "undefined") { //$NON-NLS-1$
				return null;
			}
			var groups = this.getAllGroupings(model.structure);
            dojo.forEach(groups, function(group) {
            	var idAttr = this.getAttributeNode(group, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            	if (idAttr) {
            		var key = idAttr.value;
            		if (textContent[key]) {
            			var label = this.getAttributeNode(group, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
            			label.value = textContent[key].label;
            		}
            	}
            }, this);
			
			var elements = this.getAllElements(model.structure);
			dojo.forEach(elements, function(element) {
            	var idAttr = this.getAttributeNode(element, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF);
            	if (idAttr) {
            		var key = idAttr.value;
            		if (textContent[key]) {
            			var label = this.getAttributeNode(element, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, ""); //$NON-NLS-1$
            			label.value = textContent[key].label;
            		}
            	}
            }, this);
			
			return model.structure;
		},
		
		updatePracticeStructureModel: function(practice, newGroups) {
			var structure = practice.model.structure;
			if ( !structure || typeof structure === "undefined") { //$NON-NLS-1$
				return;
			}
			var groups = this.getAllGroupings(structure);
            dojo.forEach(groups, function(group) {
            	var newCreatedAttribute = this.getAttributeNode(group, ProcessAuthoringConstants.XMLATTRIBUTE_NEW_CREATED, ""); //$NON-NLS-1$
            	if (newCreatedAttribute && newCreatedAttribute.value == ProcessAuthoringConstants.VALUE_TRUE) {
            		var idAttribute = this.getAttributeNode(group, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            		var id = idAttribute.value;
            		group.removeAttributeNode(newCreatedAttribute);
            		group.removeAttributeNode(idAttribute);
            		newGroups.push({
            			'node': group, //$NON-NLS-1$
            			'id': id //$NON-NLS-1$
            		});
            	}
            }, this);
		},
		
		getPracticeElementTextContent: function(doc) {
			var textContent = {};
			var sections = this.getAllSections(doc);
			dojo.forEach(sections, function(section) {
            	var idAttr = this.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            	if (idAttr) {
					var key = idAttr.value;
					var value = this.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
					textContent[key] = {};
					textContent[key].label = value;
				}
			}, this);
			var fields = this.getAllFields(doc);
			dojo.forEach(fields, function(field) {
            	var idAttr = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
            	if (idAttr) {
					var key = idAttr.value;
					var label = this.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
					var value = this.getFieldText(field);
					textContent[key] = {};
					textContent[key].label = label;
					textContent[key].value = value;
				}
			}, this);
			return textContent;
		},
		
		getAllSections: function(doc) {
			return this.getElementsByTagNameNS(doc, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_SECTION);
		},
		
		getAllFields: function(doc) {
			return this.getElementsByTagNameNS(doc, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_FIELD);
		},
		
		getAllGroupings: function(doc) {
			return this.getElementsByTagNameNS(doc, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_GROUPING);
		},
		
		getAllElements: function(doc) {
			return this.getElementsByTagNameNS(doc, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_ELEMENT);
		},
		
		getPracticeTextContent: function(doc) {
			var textContent = {};
			var groupings = this.getAllGroupings(doc);
			dojo.forEach(groupings, function(grouping) {
				var key = this.getAttributeNode(grouping, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF).value;
				var value = this.getAttributeNode(grouping, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
				textContent[key] = {};
				textContent[key].label = value;
			}, this);
			var elements = this.getAllElements(doc);
			dojo.forEach(elements, function(element) {
				var key = this.getResourceId(this.getAttributeNode(element, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF).value);
				var value = this.getAttributeNode(element, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
				textContent[key] = {};
				textContent[key].label = value;
			}, this);
			return textContent;
		},
		
		getAssociationTargets: function(doc) {
			var targetNodes = this.getElementsByTagNameNS(doc, ProcessAuthoringConstants.NAMESPACE_JTP, ProcessAuthoringConstants.XMLELEMENT_TARGET);
			var targets = [];
			dojo.forEach(targetNodes, function(target) {
				var targetName = this.getAttributeNode(target, ProcessAuthoringConstants.XMLELEMENT_TITLE, ProcessAuthoringConstants.NAMESPACE_DC).value;
				var targetUri = this.getAttributeNode(target, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF).value;
				targets.push({
					'name': targetName, //$NON-NLS-1$
					'target': targetUri //$NON-NLS-1$
				});
			}, this);
			return targets;
		},
		
		createElementNS: function(namespace, tagName) {
			if (document.createElementNS) {
				return document.createElementNS(namespace, tagName);
			} else {
				var doc = this.getMsxmlDOMDocument();
				var node = doc.createNode(1/*NODE_ELEMENT (1)*/, tagName, namespace);
				return node;
			}
		},
		
		createAttributeNS: function(node, namespace, name, value) {

			var attr = null;
			if (document.createAttributeNS) {
				attr = document.createAttributeNS(namespace, name);
			} else {
					var doc = this.getMsxmlDOMDocument();
					attr = doc.createNode(2/*NODE_ATTRIBUTE (2)*/, name, namespace);
                }
			
			attr.value = value;
			if (node.attributes.setNamedItemNS) {
				node.attributes.setNamedItemNS(attr);
			} else {
                try {
                	node.attributes.setNamedItem(attr);
                } catch(e) {
					if ( ns[namespace] ) {
						node.setAttribute(ns[namespace]+name, value);	
					}
				}
			}
		},
		
		getMsxmlDOMDocument: function() {
			var doc = null;
			var ms = function(n) {
				return "MSXML" + n + ".DOMDocument"; //$NON-NLS-1$ //$NON-NLS-2$
			};
			var dp = ["Microsoft.XMLDOM", ms(6), ms(4), ms(3), ms(2)]; //$NON-NLS-1$
			dojo.some(dp, function(p) {
				try {
					doc = new ActiveXObject(p);
				} catch (e) {
					return false;
				}
				return true;
			});
			
			return doc; //DOMDocument
		},
		
		/**
		 * This method is written for ContentEditor and GeneralInputFieldEditor,
		 * which should share a supertype.
		 */
		setReadOnlyWarning: function(editor) {
			var messageBoxArgs = {};
			
			editor.connect(editor.warningLink, "onclick", "onNavigateSharedPractice"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(editor.warningMessageSpan, "display", "inline"); //$NON-NLS-1$ //$NON-NLS-2$
			messageBoxArgs.summary = editor.warningMessageSpan;
			messageBoxArgs.closable = false;
			
			//show message
			messageBoxArgs.type = jazz.ui.MessageBox.STATUS_TYPES.WARNING;
			editor.messageBox.showMessage(messageBoxArgs);
		},

		getServerUrl: function() {
			var location = document.location.href;
			var separator = "/"; //$NON-NLS-1$
			var path = location.split(separator); //$NON-NLS-1$
			var serverAddress;
			if (path.length > 0) {
				serverAddress = path.shift() + separator + separator;
				path.shift(); //empty from the double //
				serverAddress = serverAddress + path.shift();
			}
			return serverAddress;
		},
		
		getContextRoot: function() {
			return net.jazz.ajax._contextRoot;
		},
		
		getProcessAuthoringRoot: function() {
			return this.getContextRoot() + "/process-authoring"; //$NON-NLS-1$
		},
		
		getURLPrefix: function() {
			return document.location.protocol + "//" + document.location.host + net.jazz.ajax._contextRoot; //$NON-NLS-1$
		},
		
		getFullProcessAuthoringRoot: function() {
			return this.getURLPrefix() + "/process-authoring"; //$NON-NLS-1$
		},
		
		isPracticeURL: function(url){
			if (url) {
				var urlPrefix = url.substring(0, url.lastIndexOf("/")); //$NON-NLS-1$
				if (urlPrefix.indexOf("/process-authoring/practices") != -1) { //$NON-NLS-1$
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		},
		
		isPracticeElementURL: function(url){
			if (url) {
				var urlPrefix = url.substring(0, url.lastIndexOf("/")); //$NON-NLS-1$
				if (urlPrefix.indexOf("/process-authoring/elements") != -1) { //$NON-NLS-1$
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		},
		
		isProcessDescriptionURL: function(url){
			if (url) {
				var urlPrefix = url.substring(0, url.lastIndexOf("/")); //$NON-NLS-1$
				if (urlPrefix.indexOf("/process-authoring/descriptions") != -1) { //$NON-NLS-1$
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		},
		
		isProcessAuthoringResourceURL: function(url) {
			if (url) {
				var urlObj = new dojo._Url(url);
				var domain = this.getMatchingAuthDomain(urlObj);
				if(domain) {
					if (domain.charAt(domain.length - 1) === '/') { //$NON-NLS-1$
						domain = domain.substring(0, domain.length - 1);
					}
					return url.indexOf(domain + ProcessAuthoringConstants.PROCESS_AUTHORING_URI_PART) != -1;
				}
			}
			return false;
		},
		
		isPracticeLibraryApp: function() {
			return net.jazz.ajax.BootstrapProperties.APP_ID == ProcessAuthoringConstants.PRACTICELIBRARY_APPID; 
		},
		
		extractETag: function(ETag) {
			if (ETag) {
				ETag = ETag.split(' '); //$NON-NLS-1$
				return {
					'modelETag': ETag[0], //$NON-NLS-1$
					'textContentETag': ETag[1] //$NON-NLS-1$
				};
			}
			
			return null;
		},
		
		getResourceId: function(resource) {
			return resource.substring(resource.lastIndexOf('/') + 1); //$NON-NLS-1$
		},
		
		//escape characters: < to &lt;
		//                   > to &gt;
		//				     " to &quot;  
		//                   ' to &#039;(Since IE doesn't support &apos; for now)
		//                   & to &amp; 
		escapeXML: function(value) {
			//don't use jazz.util.html.escape(value). This function doesn't encode character "
			if (!value) {
				return ProcessAuthoringConstants.EMPTY;
			}
			return value.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\"/g, "&quot;").replace(/\'/g, "&#039;"); //$NON-NLS-1$ //$NON-NLS-2$  //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		},
		
		unescapeXML: function(value) {
			if (!value) {
				return ProcessAuthoringConstants.EMPTY;
			}
			return value.replace(/&lt;/g, "<").replace(/&gt;/g, ">").replace(/&quot;/g, "\"").replace(/&#039;/g, "\'").replace(/&amp;/g, "&"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		},
		
		escapeHTML: function(value) {
			if (!value) {
				return ProcessAuthoringConstants.EMPTY;
			}
			return value.replace(/&/g, "&amp;").replace(/\"/g, "&quot;").replace(/\'/g, "&#039;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\ /g, "&nbsp;").replace(/\n/g, "<br>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
		},
		
		escapeHTMLWithoutSpace: function(value) {
			if (!value) {
				return ProcessAuthoringConstants.EMPTY;
			}
			return value.replace(/&/g, "&amp;").replace(/\"/g, "&quot;").replace(/\'/g, "&#039;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/\n/g, "<br>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
		},
		
		getRichText: function(text) {
            if (!text) {
            	return ProcessAuthoringConstants.EMPTY;
            }
			return this.escapeHTML(text); //$NON-NLS-1$ //$NON-NLS-2$ 
		},
		
		getPlainText: function(richText) {
			if ( !richText ) {
				return ProcessAuthoringConstants.EMPTY;
			}
			
			//<p> and <li> will not be changed to RETURN in innerText, <li> and <br> will not be changed to RETURN in textContent
			//so we need to pre-process this
            if (dojo.isIE ) {
            	richText = richText.replace(/<p>|<\/p>|<\/li>/gi, '<br/>'); //$NON-NLS-1$
            } else {
				richText = richText.replace(/<br>|<br\/>|<\/li>/gi, '\n'); //$NON-NLS-1$
			}
			var div = document.createElement(ProcessAuthoringConstants.DIV);
			dojo.html.set(div, richText);
			if (div.innerText) {
				return div.innerText;
			} else if (div.textContent) {
				return div.textContent;
			} else {
				//most likely will not go here, just for safety   
				return this._getPlainText(richText);
			}
		},

		_getPlainText: function(richText) {
			if ( !richText ) {
				return ProcessAuthoringConstants.EMPTY;
			}
            dojo.forEach(stripContentTagArray, function(tag){
            	richText = richText.replace(getRegForTag(tag, 0), ProcessAuthoringConstants.EMPTY);
            });
            dojo.forEach(singleTagArray, function(tag){
                if (tag === 'br' ) { //$NON-NLS-1$ 
                	richText = richText.replace(getRegForTag(tag, 1), '\n'); //$NON-NLS-1$
                } else {
                	richText = richText.replace(getRegForTag(tag, 1), ProcessAuthoringConstants.EMPTY);
                }
			});
            dojo.forEach(stripTagArray, function(tag){
            	richText = richText.replace(getRegForTag(tag, 2), function(match){
					return (match.toLowerCase() === "</li>" || match.toLowerCase() === '</p>')? '\n' : ProcessAuthoringConstants.EMPTY; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				});
			});
			
	     	return this.unescapeXML(richText);			
		},
		
		
		/**
		 * NOTE: copied from /net.jazz.web.app/resources/internal/ProxyRequestInterceptor.js
		 * 
		 * Compare the two dojo._Url objects' scheme, host, and port. Returns 'true' if
		 * all parts are the same (disregarding case differences), otherwise false.
		 * 
		 * @param {Object} url1
		 * @param {Object} url2
		 */
		originsAreEquivalent: function(url1, url2) {
			var props = ["scheme", "host", "port"]; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			for(var i=0; i<props.length; i++) {
				var prop = props[i];
				var val1 = url1[prop];
				var val2 = url2[prop];
				if((!val1 && val2) || (val1 && !val2)) {
					return false;
				}
				if(val1.toLowerCase() !== val2.toLowerCase()) {
					return false;
				}
			}
			return true;
		},
		
		startWith: function(str, substr) {
			var reg = new RegExp("^" + substr);  //$NON-NLS-1$   
			return reg.test(str);    
		},
		
		endWith: function(str, substr) {
			var reg=new RegExp(substr+"$");  //$NON-NLS-1$
			return reg.test(str); 
		},
		
		/**
		 * Checks the URL object's port property and if the port property is not set,
		 * sets it to either "80" (HTTP) or "443" (HTTPS).
		 * 
		 * @param {Object} url the URL object to check and perhaps normalize
		 */
		normalizePort: function(url) {
			var scheme = url.scheme || "http"; //$NON-NLS-1$
			if(!url.port) {
				url.port = scheme === "http" ? "80" : "443"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		},
		
		/**
		 * @param {Object} url
		 */
		matchesAuthDomain: function (url) {
			for (var i = 0; i < net.jazz.ajax._oauthDomains.length; i++) {
				var curDomain = new dojo._Url(net.jazz.ajax._oauthDomains[i]);
				this.normalizePort(curDomain);
				var curDomainPath = curDomain.path;
				if (this.originsAreEquivalent(url, curDomain) && (url.path.indexOf(curDomainPath) === 0))
					return true;
			}
			return false;
		},
		
		/**
		 * @param {Object} url
		 */
		getMatchingAuthDomain: function (url) {
			for (var i = 0; i < net.jazz.ajax._oauthDomains.length; i++) {
				var curDomain = new dojo._Url(net.jazz.ajax._oauthDomains[i]);
				this.normalizePort(curDomain);
				var curDomainPath = curDomain.path;
				if (this.originsAreEquivalent(url, curDomain) && (url.path.indexOf(curDomainPath) === 0))
					return net.jazz.ajax._oauthDomains[i];
			}
			return null;			
		},
		
		getLicenseWarning: function (ioArgs) {
			var warning = ioArgs.xhr.getResponseHeader("Warning"); //$NON-NLS-1$
			var licenseWarning = null;
			if(warning) {
				var splits = warning.split(" "); //$NON-NLS-1$
				if(splits.length > 2 && splits[0] == 199 && splits[1] == ProcessAuthoringConstants.WARNING_SOURCE_PROCESS_AUTHORING_LICENSE) {
					var start = warning.indexOf(ProcessAuthoringConstants.WARNING_SOURCE_PROCESS_AUTHORING_LICENSE) + ProcessAuthoringConstants.WARNING_SOURCE_PROCESS_AUTHORING_LICENSE.length + 2;
					end = warning.length - 1;
					licenseWarning = warning.substring(start, end);
					licenseWarning = decodeURI(licenseWarning);
				}
			}
			return licenseWarning;
		}
	}
	
})();
