/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.AllReportTemplatesTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.app.context");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.SimpleToolbar");

dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var session = com.ibm.team.repository.web.client.session;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var HTML = jazz.util.html;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.ui.ResourceLink;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var AllReportTemplatesTab = dojo.declare("com.ibm.team.reports.web.ui.internal.AllReportTemplatesTab",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/AllReportTemplatesTab.html"),
	_csPageHelpWidgets: null,
	csHelpNode: null,
	_pendingDeleteNode: null,
	
	constructor: function(args) {
		this.templatesHeading = messages.reportTemplates;
		this._csPageHelpWidgets= [];
	},
	
	postCreate: function() {
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._table = new StyledTable({columns: [{label: messages.reportTemplate, width: "80%", sortable: false}, {label: messages.actions, width: "20%", sortable: false}], 
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, showHover: true});
		this._resultsArea.appendChild(this._table.domNode);
		
		srh = new ServiceResponseHandler(this, "_customReportingConfigured", "_handleCustomReportingConfiguredError");
		ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.reports.service.cognos.internal.CognosReportService");
		
		this._createContextSensitiveHelp();
		this._runQuery();
		
		this.messageBox = new jazz.ui.MessageBox();
		this._messageBoxContainer.appendChild(this.messageBox.domNode);
	},
	
	_createContextSensitiveHelp: function() {
		dojo.empty(this.csHelpNode);
		this._createCSHWidget(AllReportTemplatesTab.CSH_ALL_REPORT_TEMPLATES);
	},

	_createCSHWidget: function(uaKey) {
		var parms = {};
		parms.uaKey = uaKey;
		var pageHelp = new PageHoverHelp(parms);
		this.csHelpNode.appendChild(pageHelp.domNode);
		this._csPageHelpWidgets.push(pageHelp);
	},

	_newTemplate: function() {
		document.location.href = "#action=com.ibm.team.reports.editTemplate";
	},
	
	_newCustomTemplate: function() {
		document.location.href = "#action=com.ibm.team.reports.editCustomTemplate";
	},
	
	_deployNewTemplates: function() {
		// Prompt the user to select a project area or team area
		var projectArea = jazz.app.context.get();
		var self = this;
		var params = {
			projects: true,
			teams: true,
			allowsMultiple: false,
			selection: projectArea.itemId,
			filterProjectArea: projectArea.itemId,
			onOk: function(selectedItem, action) {
				self._onProcessOk(selectedItem);
			}
		};
		new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
	},
	
	_onProcessOk: function(selectedItem) {
		// Load the deployNewTemplates page
		document.location.href = "#action=com.ibm.team.reports.deployNewTemplates&processAreaUUID=" + selectedItem.itemId;
	},

	_customReportingConfigured: function(response) {
		for(var i = 0; i < response.properties.length; i++){
			if(response.properties[i].name == "cognos.hostAndPort"){
				if(response.properties[i].overrideValue && response.properties[i].overrideValue.length > 0){
					this._buildToolbar(true);
					return;
				}
			}
		}
		this._buildToolbar(false);
	},
	
	_handleCustomReportingConfiguredError: function() {
		this._buildToolbar(false);
	},
	
	_buildToolbar: function(showCustom){
		var toolbar = new jazz.ui.SimpleToolbar({});
		this._toolbarDiv.appendChild(toolbar.domNode);
		toolbar.addButton(this, this._newTemplate, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-template_obj.gif"), label: messages.createNewTemplate});
		if(showCustom){
			toolbar.addButton(this, this._newCustomTemplate, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-custom-report-template_co.gif"), label: messages.createNewTemplateFromCustomReport});
		}
		toolbar.addButton(this, this._deployNewTemplates, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/find-template_obj.gif"), label: messages.deployNewTemplates});
		
	},
	
	_runQuery: function() {
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var serviceObject = {
			self: this,
			id: this._nextQueryId,
			success: function(results) {
				if (this.self._lastQueryId && this.self._lastQueryId > this.id)
					return;
				this.self._lastQueryId = this.id;
				this.self._renderResults(results);
				PlatformUI.getWorkbench().hideMessageArea();
			},
			failure: function(error) {
				// TODO
				//dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		var args = {
			sortByName: true
		};
		if (this._currentFilter) {
			args.filter = this._currentFilter;
		}
		ReportsClient.getReportDescriptors(srh, args);
		this._nextQueryId++;
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.length > 0) {
			for (var i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createTemplateRow(results[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	_createTemplateRow: function(template) {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.innerHTML = "<td><a href='#action=com.ibm.team.reports.editTemplate&reportUUID=" + template.reportUUID + "'>" + HTML.escape(template.name) + "</a>";
		row.id = template.reportUUID;
		row.appendChild(td);
		
		// Edit template, delete template
		td = document.createElement("td");
		var div = document.createElement("div");
		div.className = "actionBar";
		
		var self = this;
		
		// Edit
		var anchor = document.createElement("a");
		anchor.href = "#";
		anchor.title = messages.edit;
		div.appendChild(anchor);
		var img = document.createElement("img");
		img.alt = messages.edit;
		img.src = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/edit.gif");
		anchor.appendChild(img);
		var handler = dojo.connect(anchor, "onclick", function(event) {
			dojo.stopEvent(event);
			self._editTemplate(template);
		});
		
		// Delete
		anchor = document.createElement("a");
		anchor.href = "#";
		anchor.title = messages.deleteMsg;
		div.appendChild(anchor);
		img = document.createElement("img");
		img.alt = messages.deleteMsg;
		img.src = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/delete.gif");
		anchor.appendChild(img);
		var handler = dojo.connect(anchor, "onclick", function(event) {
			dojo.stopEvent(event);
			self._deleteTemplate(template);
		});

		td.appendChild(div);
		row.appendChild(td);
		
		return row;
	},
	
	_editTemplate: function(template) {
		document.location.href = "#action=com.ibm.team.reports.editTemplate&reportUUID=" + template.reportUUID;
	},
	
	_deleteTemplate: function(template) {
		this._clearStatusMessage();
		if (!confirm(messages.confirmDelReport)) {
			return;
		}
		var args = {
    		reportUUID: template.reportUUID
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_templateDeleted",
				"_remoteError");
		var rowNode = dojo.query("tr[id=" + template.reportUUID + "]");
		if(rowNode.length > 0){
			rowNode = rowNode[0];
			this._pendingDeleteNode = rowNode;
		}
		ReportsClient.postDeleteReportDescriptor(responseHandler, args);
	},
	
	_templateDeleted: function() {
		this._showStatusMessage(messages.deletionSuccessful);
		if(this._pendingDeleteNode && this._pendingDeleteNode != null){
			this._table.tbody.removeChild(this._pendingDeleteNode);
			this._pendingDeleteNode = null;
		}
	},
	
	_createNoneMessage: function() {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.innerHTML = HTML.escape(messages.allTemplatesNone);
		row.appendChild(td);
		return row;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterStale = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (!this._runQueryTimeout) {
			this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runQuery,0,true),500);
		}
	},
	
	_remoteError: function(error) {
		//alert(error.message);
	},
	
	_showStatusMessage: function(message) {
		this._clearStatusMessage();
		var status = {
			type : jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary : message,
			closable : false
		};
		dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
		this.messageBox.showMessage(status);
	},
	
	_clearStatusMessage: function() {
		dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
		this.messageBox.hideMessage();
	}	
});

AllReportTemplatesTab.CSH_ALL_REPORT_TEMPLATES = "com.ibm.jazz.reports.web.cshelp.all_report_templates";

})();
