/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.AllReportsTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.SimpleToolbar");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {

var session = com.ibm.team.repository.web.client.session;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var HTML = jazz.util.html;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.ui.ResourceLink;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var BASE_PADDING = 9;
var INDENT = 38;

var AllReportsTab = dojo.declare("com.ibm.team.reports.web.ui.internal.AllReportsTab",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/AllReportsTab.html"),
	_csPageHelpWidgets: null,
	csHelpNode: null,
	_pendingDeleteReportNode: null,
	_pendingDeleteFolderNode: null,
	_pendingDeleteFolderChildrenNodes: null,
	_childMap: {},
	
	constructor: function(args) {
		this.messages = messages;
		if (args.shared == "true") {
			this.allHeading = messages.sharedReports;
		} else {
			this.allHeading = messages.myReports;
		}
		this._controller = args.controller;
		this._csPageHelpWidgets= [];
	},
	
	postCreate: function() {
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._table = new StyledTable({columns: [{label: messages.report, width: "90%", sortable: false}, {label: messages.actions, width: "10%", 
			className: "action-column", sortable: false}], trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, showHover: true});
		this._resultsArea.appendChild(this._table.domNode);
		
		var toolbar = new jazz.ui.SimpleToolbar({});
		this._toolbarDiv.appendChild(toolbar.domNode);
		toolbar.addButton(this, this._newFolder, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-folder-obj.gif"), label: messages.createFolder});
		this._createContextSensitiveHelp();
		this._runNewQuery();
		dojo.connect(this._folderFilterSelect, "onchange",this, this._runNewQueryForView);
	},
	
	_runNewQueryForView: function(event){
		var select = this._folderFilterSelect;
		var folderUUID = select.options[select.selectedIndex].value;
		if(folderUUID){
			if(folderUUID == "_allFolders"){
				this._runNewQuery();
			}else{
				this._renderChildFoldersForParent(folderUUID);
			}
		}
	},
	
	_createContextSensitiveHelp: function() {
		dojo.empty(this.csHelpNode);
		if (this.shared == "true") {
			this._createCSHWidget(AllReportsTab.CSH_SHARED_REPORTS);
		} else {
			this._createCSHWidget(AllReportsTab.CSH_MY_REPORTS);
		}
	},

	_createCSHWidget: function(uaKey) {
		var parms = {};
		parms.uaKey = uaKey;
		var pageHelp = new PageHoverHelp(parms);
		this.csHelpNode.appendChild(pageHelp.domNode);
		this._csPageHelpWidgets.push(pageHelp);
	},
	
	_newFolder: function() {
		var isShared = (this.shared == "true") ? true : false;
		document.location.href = "#action=com.ibm.team.reports.editFolder&shared=" + isShared;
	},
	
	_runNewQuery: function() {
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var responseHandler = new ServiceResponseHandler(
				this,
				"_renderRootFolders",
				"_remoteError");
		ReportsClient.getSystemRootFolderDescriptors(responseHandler);
	},
	
	/* Deprecated */
	_runQuery: function() {
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var serviceObject = {
			self: this,
			id: this._nextQueryId,
			success: function(results) {
				if (this.self._lastQueryId && this.self._lastQueryId > this.id)
					return;
				this.self._lastQueryId = this.id;
				this.self._renderResults(results);
				PlatformUI.getWorkbench().hideMessageArea();
			},
			failure: function(error) {
				// TODO
				//dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		var args = {
			// Project Area will be added by ReportsClient
    		sortByName: true
    	};
    	if (this._currentFilter) {
    		args.filter = this._currentFilter;
    	}
    	if (this.shared == "true") {
    		args.flags = ReportsClient.EXCLUDE_MICRO | ReportsClient.SHARED;
    	} else {
    		args.flags = ReportsClient.EXCLUDE_MICRO | ReportsClient.PRIVATE;
    		args.ownerUUID = session.getAuthenticatedContributor().itemId;
    	}
		ReportsClient.getQueryDescriptors(srh, args);
		this._nextQueryId++;
	},
	
	_renderRootFolders: function(results) {
		// Don't actually create table nodes for the root "My Reports" or "Shared Reports".
		// Just do the recursive call.
		if (results.length != 2) {
			// Something went wrong
			return;
		}
		var index = (this.shared == "true") ? 1 : 0;
		this._rootFolder = results[index];
		this._renderChildFoldersForParent(this._rootFolder.folderUUID);
	},
	
	_renderChildFoldersForParent: function(folderUUID){
		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		this._table.clearBody();
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var args = {
			// Project Area will be added by ReportsClient
			// Owner is intentionally empty
    		parentUUID: folderUUID,
    		flags: ReportsClient.ALL,
    		sortByName: true
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_addChildFolders",
				"_remoteError");
		ReportsClient.getFolderDescriptors(responseHandler, args);
		var args2 = {
			// Project Area will be added by ReportsClient
			// Owner is intentionally empty
    		folderUUID: folderUUID,
    		flags: ReportsClient.EXCLUDE_MICRO,
    		sortByName: true
    	};
    	if (this._currentFilter) {
    		args2.filter = this._currentFilter;
    	}
		var responseHandler2 = new ServiceResponseHandler(
				this,
				"_addChildQueries",
				"_remoteError");
		ReportsClient.getQueryDescriptors(responseHandler2, args2);
	},
	
	_addChildFolders: function(results) {
		PlatformUI.getWorkbench().hideMessageArea();
		if (results != null) {
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				this._folderFilterSelect.options[i + 1] = new Option(HTML.escape(folder.name), folder.folderUUID);
				this._childMap[folder.folderUUID] = new Array();
				// Locate the parent to insert it under
				if (folder.parentUUID == this._rootFolder.folderUUID || folder.parentUUID == null) {
					this._table.tbody.appendChild(this._createFolderRow(folder, 9));
				} else {
					var parent = document.getElementById(folder.parentUUID);
					if (parent == null) {
						// Do nothing, shouldn't happen.
					} else {
						this._insertAfter(this._createFolderRow(folder, parent._indent + INDENT), parent);
					}
				}
			}
			// Recurse
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				var args = {
					// Project Area will be added by ReportsClient
					// Owner is intentionally empty
	        		parentUUID: folder.folderUUID,
	        		flags: ReportsClient.ALL,
	        		sortByName: true
	        	};
				var responseHandler = new ServiceResponseHandler(
						this,
						"_addChildFolders",
						"_remoteError");
				ReportsClient.getFolderDescriptors(responseHandler, args);
				var args2 = {
					// Project Area will be added by ReportsClient
					// Owner is intentionally empty
	        		folderUUID: folder.folderUUID,
	        		flags: ReportsClient.EXCLUDE_MICRO,
		    		sortByName: true
	        	};
		    	if (this._currentFilter) {
		    		args2.filter = this._currentFilter;
		    	}
				var responseHandler2 = new ServiceResponseHandler(
						this,
						"_addChildQueries",
						"_remoteError");
				ReportsClient.getQueryDescriptors(responseHandler2, args2);
			}
		}
	},
	
	_addChildQueries: function(results) {
		PlatformUI.getWorkbench().hideMessageArea();
		if (results != null) {
			for (var i = 0; i < results.length; i++) {
				var query = results[i];
				this._childMap[query.folderUUID] = new Array();
				this._childMap[query.folderUUID].push(query.queryUUID);
				// Locate the parent to insert it under
				if (query.folderUUID == this._rootFolder.folderUUID) {
					this._table.tbody.appendChild(this._createQueryRow(query, 9));
				} else {
					var parentFolder = document.getElementById(query.folderUUID);
					if (parentFolder == null) {
						// Couldn't find the folder, so must be a root folder
						this._table.tbody.appendChild(this._createQueryRow(query, 9));
					} else {
						//this._insertAfter(this._createQueryRow(query, parentFolder._indent + INDENT), parentFolder);
						this._insertAfter(this._createQueryRow(query, parentFolder._indent + INDENT), this._getQueryInsertionPoint(query.folderUUID));
					}
				}
			}
		}
	},
	
	_getQueryInsertionPoint: function(folderUUID) {
		var folderTRNode = document.getElementById(folderUUID);
		var candidate = folderTRNode;
		var nextCandidate = candidate.nextSibling;
		while (true) {
			if (nextCandidate == null) {
				return candidate;
			}
			if (nextCandidate._indent == folderTRNode._indent) {
				return candidate;
			}
			candidate = nextCandidate;
			nextCandidate = nextCandidate.nextSibling;
		} 
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.length > 0) {
			for (var i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createQueryRow(results[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	_createQueryRow: function(report, padding) {
		var row = document.createElement("tr");
		row._indent = padding;
		var td = document.createElement("td");
		td.innerHTML = "<td><a href='#action=com.ibm.team.reports.viewQuery&queryUUID=" + report.queryUUID + "'>" + HTML.escape(report.name) + "</a>";
		td.style.paddingLeft = padding + "px";
		td.className="reportName";
		row.appendChild(td);
		row.id = report.queryUUID;
		
		td = document.createElement("td");
		td.className = "action-column";
		var toolbar = new com.ibm.team.reports.web.ui.internal.common.Toolbar({});
		toolbar.addButton(this, dojo.hitch(this, "_renameReport", report), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/rename.gif"), label: messages.rename});
		toolbar.addButton(this, dojo.hitch(this, "_deleteReport", report), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/delete.gif"), label: messages.deleteMsg});
		td.appendChild(toolbar.domNode);
		
		row.appendChild(td);
		return row;
	},
	
	_createFolderRow: function(folder, padding) {
		var row = document.createElement("tr");
		row.setAttribute("id", folder.folderUUID);
		row._indent = padding;
		var td = document.createElement("td");		
		td.innerHTML = "<td>" + HTML.escape(folder.name);
		td.style.paddingLeft = padding + "px";
		td.className="reportName";
		row.appendChild(td);
		row.id = folder.folderUUID;
		
		td = document.createElement("td");
		td.className = "action-column";
		var toolbar = new com.ibm.team.reports.web.ui.internal.common.Toolbar({});
		toolbar.addButton(this, dojo.hitch(this, "_createNewFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/new-folder-obj.gif"), label: messages.createFolder});
		toolbar.addButton(this, dojo.hitch(this, "_editFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/edit.gif"), label: messages.edit});
		toolbar.addButton(this, dojo.hitch(this, "_deleteFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/delete.gif"), label: messages.deleteMsg});
		td.appendChild(toolbar.domNode);
		
		row.appendChild(td);
		return row;
	},
	
	_createNewFolder: function(folder) {
		document.location.href = "#action=com.ibm.team.reports.editFolder&parentUUID=" + folder.folderUUID + "&processAreaUUID=" + folder.processAreaUUID + "&shared=" + folder.shared;
	},
	
	_editFolder: function(folder) {
		document.location.href = "#action=com.ibm.team.reports.editFolder&folderUUID=" + folder.folderUUID;
	},
	
	_deleteFolder: function(folder) {
		if (!confirm(messages.confirmDelFolder)) {
			return;
		}
		var args = {
    		folderUUID: folder.folderUUID
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_folderDeleted",
				"_remoteError");
		var rowNode = dojo.query("tr[id=" + folder.folderUUID + "]");
		if(rowNode.length > 0){
			rowNode = rowNode[0];
			this._pendingDeleteFolderNode = rowNode;
		}
		//find all of the child nodes
		this._pendingDeleteFolderChildrenNodes = new Array();
		this._addAllChildReportsToFolderDelete(folder.folderUUID);
		ReportsClient.postDeleteFolderDescriptor(responseHandler, args);
	},
	
	_addAllChildReportsToFolderDelete: function(uuid){
		var folderChildren = this._childMap[uuid];
		if(folderChildren){
			for(var i = 0; i < folderChildren.length; i++){
				var child = dojo.query("tr[id=" + folderChildren[i] + "]");
				if(child.length > 0){
					this._pendingDeleteFolderChildrenNodes.push(child[0]);
					this._addAllChildReportsToFolderDelete(child[0].id);
				}
			}
		}
	},
	
	_folderDeleted: function() {
		if(this._pendingDeleteFolderNode && this._pendingDeleteFolderNode != null){
			this._table.tbody.removeChild(this._pendingDeleteFolderNode);
			this._pendingDeleteFolderNode = null;
		}
		if(this._pendingDeleteFolderChildrenNodes && this._pendingDeleteFolderChildrenNodes != null){
			for(var i = 0; i < this._pendingDeleteFolderChildrenNodes.length; i++){
				this._table.tbody.removeChild(this._pendingDeleteFolderChildrenNodes[i]);
			}
			this._pendingDeleteFolderChildrenNodes = null;
		}
	},

	_renameReport: function(report) {
		var self = this;
		
		var dialogContentDiv = document.createElement("div");
		dialogContentDiv.className = "com-ibm-team-reports";
		
		var messageBox = new jazz.ui.MessageBox();
		dialogContentDiv.appendChild(messageBox.domNode);
		
		var newNameLabel = document.createElement("label");
		newNameLabel.className = "newQueryNameLabel";
		newNameLabel.appendChild(document.createTextNode(messages.newName));
		var newNameTextField = document.createElement("input");
		newNameTextField.type = "text";
		newNameLabel.appendChild(newNameTextField);
		
		var buttonDiv = document.createElement("div");
		buttonDiv.className = "newQueryNameButtons";
		var okBtn = document.createElement("button");
		okBtn.setAttribute(dojo.isIE ? "cssText" : "style", "margin-right: 5px;");
		okBtn.innerHTML = messages.ok;
		okBtn.onclick = function() {
			var newName = self._trim(newNameTextField.value);
			if (newName.length == 0) {
				var status = {
					type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary : self.messages.invalidNameEntered,
					closable : false
				};
				messageBox.showMessage(status);
				return;
			}
			self._newName = newName;
			var args = {
    			queryUUID: report.queryUUID,
    			name: newName
    		};
			var responseHandler = new ServiceResponseHandler(
				self,
				"_queryRenamed",
				"_remoteError");
			ReportsClient.postRenameQueryDescriptor(responseHandler, args);
			self._dialog.close();
			self._dialog.destroy();
		}
		
		var cancelBtn = document.createElement("button");
		cancelBtn.innerHTML = "Cancel";
		cancelBtn.onclick = function() {
			self._dialog.close();
			self._dialog.destroy();
		}
		buttonDiv.appendChild(okBtn);
		buttonDiv.appendChild(cancelBtn);
		
		dialogContentDiv.appendChild(newNameLabel);
		dialogContentDiv.appendChild(buttonDiv);
		
		this._dialog = new jazz.ui.Dialog({
			contentNode: dialogContentDiv,
			primaryTitle: messages.rename,
			width: "300px",
			isClosable: true
		});
		
		newNameTextField.focus();
	},
	
	_queryRenamed: function() {
		//if (this._newName != null) {
		//	this._textNode.nodeValue = this._newName;
		//}
		//this._newName = null;
	},
	
	_deleteReport: function(report) {
		if (!confirm(messages.confirmDelReport)) {
			return;
		}
		var args = {
    		queryUUID: report.queryUUID
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_queryDeleted",
				"_remoteError");
		var rowNode = dojo.query("tr[id=" + report.queryUUID + "]");
		if(rowNode.length > 0){
			rowNode = rowNode[0];
			this._pendingDeleteReportNode = rowNode;
		}
		ReportsClient.postDeleteQueryDescriptor(responseHandler, args);
	},
	
	_queryDeleted: function() {
		if(this._pendingDeleteReportNode && this._pendingDeleteReportNode != null){
			this._table.tbody.removeChild(this._pendingDeleteReportNode);
			this._pendingDeleteReportNode = null;
		}
	},
	
	_createNoneMessage: function() {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.innerHTML = HTML.escape(messages.allReportsNone);
		row.appendChild(td);
		return row;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterStale = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (!this._runQueryTimeout) {
			this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runNewQueryForView,0,true),500);
		}
	},
	
	_remoteError: function(error) {
		//alert(error.message);
	},
	
	_insertAfter: function(newNode, referenceNode) {
	    referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
	},
	
	_trim: function(string) {
		return this._trimLeft(this._trimRight(string));
	},
	_trimLeft: function(string) {
		return string.replace(new RegExp("^[\\s]+", "g"), "");
	},
	_trimRight: function(string) {
		return string.replace(new RegExp("[\\s]+$", "g"), "");
	}
	
});

AllReportsTab.CSH_SHARED_REPORTS = "com.ibm.jazz.reports.web.cshelp.shared_reports";
AllReportsTab.CSH_MY_REPORTS = "com.ibm.jazz.reports.web.cshelp.my_reports";

})();
