/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.viewlets.cognos.ui.internal.CognosReportsViewlet");

dojo.require("com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");

dojo.require("com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");

dojo.require("jazz.client.xhr");
dojo.require("jazz.util.i18n");

dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.reports.viewlets.cognos.ui.internal", "CognosReportsViewletMessages");

(function() {

var CognosReportsBrowseDialog = com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog;
var CognosReportsClient = com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var I18nUtil = jazz.util.i18n;
var messages;
var GET_CAN_RUN_CUSTOM_REPORT_SERVICE_URL = "service/com.ibm.team.reports.common.internal.service.IReportRestService/canRunCustomReport";
var MAX_WAIT_SECONDS = 5;
var TIMEOUT_INTERVAL = 250;
var COOKIE_RESET_INTERVAL = 1800000; //30 minutes

/*
 * The Cognos viewlet class.
 */
dojo.declare("com.ibm.team.reports.viewlets.cognos.ui.internal.CognosReportsViewlet",
	com.ibm.team.dashboard.web.ui.Viewlet, {
	
	_height: null,
	_width: null,
	_heightInput: null,
	_widthInput: null,
	_JtsUrl: null,
	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.viewlets.cognos", "ui/internal/templates/CognosReportsViewlet.html"),
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.viewlets.cognos.ui.internal", "CognosReportsViewletMessages");
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
		this.initStaticFunctions();
		setTimeout(dojo.hitch(this, "_resetCookieFunc"), COOKIE_RESET_INTERVAL);
	},
	
	initStaticFunctions: function() {
		//set up cookie handling
		var cookieQueue = function cookieQueue(getCookieCallback, runReportCallback) {
			if (window.waitingQueue === undefined) {
				window.waitingQueue = new Array();
			} 
			//Establish the cookie if not present
			if (window.cookiePresent === undefined) {
				window.cookiePresent = false;
				window.waitingQueue.push(runReportCallback)
				getCookieCallback();
			} else if (window.cookiePresent == false) {
				window.waitingQueue.push(runReportCallback);
			} else {
				//if the cookie has been established already, go ahead and run
				runReportCallback();
			}
		}
		var cookieReceived = function RRDICookieReceived() {
			window.cookiePresent = true;
			//once we have the cookie, run all the callbacks that registered
			while (window.waitingQueue.length > 0) {
				var aRunReportCallback = window.waitingQueue.shift();
				aRunReportCallback();
			}
		}
		
		if (!window.RRDICookieQueue) { 
			window.RRDICookieQueue = cookieQueue; 
		}
		if (!window.RRDICookieReceived) { 
			window.RRDICookieReceived = cookieReceived; 
		}
	},
	
	init: function() {
		var srh = new ServiceResponseHandler(this, "_handleFetchJTSUrl", "_handleFetchJTSUrlError");
		ServerStatusClient.getJazzTeamServerURI(srh);
		srh = new ServiceResponseHandler(this, "_handleGetCustomReportingUrl", "_handleGetCustomReportingUrlError");
		ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.reports.service.cognos.internal.CognosReportService");
		dojo.connect(this.getViewletState(), "stopEditing", dojo.hitch(this, "_stopEditing"));	
		// Local vs. Remote viewlet flag
		if (dojo.getObject("com.ibm.team.process.web.projectArea")) {
			this._local = true;
		} else {
			this._local = false;
		}
	 
		var nodes = I18nUtil.addMessage(this._noQueriesEditContentsLink, this.messages.msgToAddACognosReport);
		this.connect(nodes[0], "onclick", this._doEdit);
		//this.connect(this.reportFrame,"onload",this._resize);
	},
	
	_handleFetchJTSUrl : function(response){
		this._JtsUrl = response;
		if(this._JtsUrl.indexOf("/", this._JtsUrl.length - 1) == -1){
			this._JtsUrl += "/";
		}
	},
	
	_handleFetchJTSUrlError : function(){
		this._JtsUrl = -1;
	},
	
	_handleGetCustomReportingUrl: function(response) {
		for(var i = 0; i < response.properties.length; i++){
			if(response.properties[i].name == "cognos.hostAndPort"){
				if(response.properties[i].overrideValue && response.properties[i].overrideValue.length > 0){
					this._customReportingUrl = response.properties[i].overrideValue;
					if(this._customReportingUrl.indexOf("/", this._customReportingUrl.length - 1) == -1){
						this._customReportingUrl += "/";
					}
					return;
				}
			}
		}
		this._customReportingUrl = -1;
	},
	
	_handleGetCustomReportingUrlError: function() {
		this._customReportingUrl = -1;
	},
	
	// Make sure uninitialize doesn't call destroy back again and result in recursive destroys
	destroy: function() {
		if (this.isDestroyed() == false)
			this.inherited(arguments);
	},
	
	// Destroy all of the widgets used by this widget.
	uninitialize: function() {
		if (this.domNode != null)
			this.destroyWidgetsFromNode(this.domNode);  
	},

    // Summary:
    //    Given a dom node, find all dijit widgets that are currently attached to
    //    it and its children nodes, and destroy all found widgets. This is useful
    //    when an UI is updating its display by recreating certain part of the dom
    //    tree and ensures that no widget's leaked due to dynamic removal.
    // 
    destroyWidgetsFromNode: function(node) {
        var list = dojo.query('[widgetId]', node);
        var removeWidgets = list.map(dijit.byNode);
        for(var i = 0; i < removeWidgets.length; i++){
        	if(removeWidgets[i].destroy){
        		removeWidgets[i].destroy();
        	}
        }
    },

	_resize: function() {
		var h = this.reportFrame.contentDocument.body.scrollHeight;
		var w = this.reportFrame.contentDocument.body.scrollWidth;
		console.debug(h+","+w);
		this.reportFrame.style.height = h +'px';
		this.reportFrame.style.width = w +'px';
	},
	
	refresh: function() {
		this.update();
		this.getSite().doneRefresh();
	},
	
	update: function(){
		if(this._JtsUrl === null){ //wait until we have the jts url to proceed
			setTimeout(dojo.hitch(this, "update"), TIMEOUT_INTERVAL);
			return;
		}
		var iFrame = this.getViewletState().get("iFrame");
		if(iFrame != null){
			//non empty viewlet
			window.RRDICookieQueue(dojo.hitch(this, "_createCognosSession", iFrame),
				dojo.hitch(this, "_createIframe", iFrame));
		}
		
		if (this.isEditing()) {// check if we're in the editing mode
			var addLi = document.createElement("div");
			addLi.className = "action add-cognos-report";
			var table = document.createElement("table");
			this._sizeTable = table;
			var row1 = document.createElement("tr");
			var cell11 = document.createElement("td");
			var cell12 = document.createElement("td");
			var row2 = document.createElement("tr");
			var cell21 = document.createElement("td");
			var cell22 = document.createElement("td");
			row1.appendChild(cell11);
			row1.appendChild(cell12);
			table.appendChild(row1);
			row2.appendChild(cell21);
			row2.appendChild(cell22);
			table.appendChild(row2);
			cell11.appendChild(document.createTextNode(this.messages.height));
			cell21.appendChild(document.createTextNode(this.messages.width));
			var heightInput = document.createElement("input");
			var widthInput = document.createElement("input");
			this._heightInput = heightInput;
			this._widthInput = widthInput;
			heightInput.style.width = '50px';
			widthInput.style.width = '50px';
			cell12.appendChild(heightInput);
			cell22.appendChild(widthInput);
			this._site.getActionArea().appendChild(table);
			var height = this.getViewletState().get("height");
			var width = this.getViewletState().get("width");
			heightInput.value = height && height != null ? height : '';
			widthInput.value = width && width != null ? width : '';
			
			heightInput.onchange = dojo.hitch(this, function(){
				this._height = heightInput.value;
			});
			widthInput.onchange = dojo.hitch(this, function(){
				this._width = widthInput.value;
			});
		}
		var length = this._site.getActionArea().childNodes.length;
		if(this._site.getActionArea().childNodes != null && !this.isEditing() && length != 0){
			while(this._site.getActionArea().firstChild){
				this._site.getActionArea().removeChild(this._site.getActionArea().firstChild);
			}
		}
		if (this.iFrame == null || this.iFrame == "") {
			var nodes = I18nUtil.addMessage(this._noQueriesEditContentsLink, this.messages.configureThisReport);
			this.connect(nodes[0], "onclick", this._addCognosReport);
			nodes[0].title = this.messages.configureThisReportLabel;
			dojo.addClass(this._noQueriesEditContentsLink, "title");
		}	
	},
	
	_onOkClick: function(){
		var reportUUID = this.cognosReportsBrowseDialog.reportUUID;
		var args ={reportUUID: reportUUID, projectAreaUUID: this.cognosReportsBrowseDialog.reportProjectAreaUUID};
		var handler = new ServiceResponseHandler(this, "_success", "_failure");
		ReportsClient.getRenderedReport(handler, args);
	},
	
	_success: function(chartHTML){
		chartHTML= chartHTML.replace("(h>600?h:600)", "h");
		var obj = new Object();
		obj.iFrame = chartHTML;
		window.RRDICookieQueue(dojo.hitch(this, "_createCognosSession", obj),
			dojo.hitch(this, "_createIframe", obj));
		this._saveChanges();		
	},
	
	_failure: function(){
		alert(this.messages.msgErrorFetchingTheProject);
	},
	
	_doEdit: function(e){
		e.preventDefault();
		this.getSite().editContents();
	},
	
	_addCognosReport: function(e){
		e.preventDefault();
		 var args = {
		 	projectAreaUUID: this._getProjectAreaUUID(),
			elementsToBeDisplayed: "Reports"
	  	};
		this.cognosReportsBrowseDialog = new CognosReportsBrowseDialog(args);
		this.connect(this.cognosReportsBrowseDialog, "onReportSelected", this._onOkClick);
	},
	
	setDashboardEditing: function(editing) {
		if (this.isEditing() || !editing) {
			dojo.addClass(this._noQueriesEditContentsLink, "hidden");
		} else {
			dojo.removeClass(this._noQueriesEditContentsLink, "hidden");
		}
	},
	
	_createIframe: function(iFrameElement, attempts){	
		this.iFrame = iFrameElement;
		var newNode = document.createElement('div');
		if(iFrameElement.iFrame) {
			var height = this.getViewletState().get("height");
			var width = this.getViewletState().get("width");
			if(height && width && height != null && width != null){
				height = height.replace(/^\s+|\s+$/g,"");  //trim
				width = width.replace(/^\s+|\s+$/g,"");   //trim
				if(!/px$/.test(height) && !/%$/.test(height)){
					height += "px";
				}
				if(!/px$/.test(width) && !/%$/.test(width)){
					width += "px";
				}
				iFrameElement.iFrame = iFrameElement.iFrame.replace(/height:.*?;/im, "height:" + height + ";");
				iFrameElement.iFrame = iFrameElement.iFrame.replace(/width:.*?;/im, "width:" + width + ";");
			}
			newNode.innerHTML = iFrameElement.iFrame;
			if(!height || !width || height == null || width == null){
				var frameHeight = newNode.firstChild.style.height;
				var frameWidth = newNode.firstChild.style.width;
				this.getViewletState().set("height", frameHeight != null ? frameHeight : "");
				this.getViewletState().set("width", frameWidth != null ? frameWidth : "");
				if(this.isEditing() && this._heightInput && this._heightInput != null && this._widthInput && this._widthInput != null){
					if(!this._heightInput.value || this._heightInput.value.replace(/^\s+|\s+$/g,"").length == 0){
						this._heightInput.value = frameHeight;
					}
					if(!this._widthInput.value || this._widthInput.value.replace(/^\s+|\s+$/g,"").length == 0){
						this._widthInput.value = frameWidth;
					}
				}
			}
		}
		this._removeChildren(this.renderedReportDiv);
		this.renderedReportDiv.appendChild(newNode);

		this._noQueriesEditContentsLink.innerHTML = '';
	},
	
	_createCognosSession: function(iFrameElement, attempts){
		var self = this;
		if(this._customReportingUrl === null && (!attempts || attempts < (MAX_WAIT_SECONDS * 1000 / TIMEOUT_INTERVAL))){ //wait until we have the custom reporting url
			if(!attempts){
				attempts = 0;
			}
			attempts += 1;
			setTimeout(dojo.hitch(this, "_createCognosSession", iFrameElement, attempts), TIMEOUT_INTERVAL);
			return;
		}
		if(this._customReportingUrl == -1){ //something went wrong fetching the custom reporting url so just try to run the report
			//since we are going to go ahead and run the report anyways, signal cookie creation otherwise we could get into an infinite wait state
			window.RRDICookieReceived();
			return;
		}
		jazz.client.xhrGet
		({
		  url: this._customReportingUrl + "?b_action=xts.run&m=portal/close.xts",
		  headers: this.headers,
		  preventCache: false,
		  load: dojo.hitch(this, function(response, ioArgs){
			  window.RRDICookieReceived();
		}),
		  error: dojo.hitch(self, function(){
			window.RRDICookieReceived();
		}),
		  handleAs: "text"
		});
	},
	
	_getValue: function(xmlText) {
		var indexOfProperty = xmlText.indexOf("<value>");
		if (indexOfProperty == -1) return null;
		xmlText = xmlText.substr(indexOfProperty + 7);
		xmlText = xmlText.substr(0, xmlText.indexOf("</value>"));
		return xmlText;
	},
	
	_saveChanges: function() {
		// if this is a new iframe, assign an id
		var iFrame = this.iFrame;
		if (iFrame != null) {
			iFrame.id = 1;
			this.getViewletState().set("iFrame", iFrame);
		}
		this.update();
		if (this._sizeTable) {
			this._site.getActionArea().removeChild(this._sizeTable);
			this._sizeTable = undefined;
		}
	},
	
	/* object */ getDefaultState: function() {
		var state = new Object();
		state.nextId = 1;
		state.iFrame = "";
		return state;
	},
	
	_removeChildren: function(element) {
			while (element.firstChild) {
				element.removeChild(element.firstChild);
			}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getProjectAreaUUID: function() {
		var projectArea = this._getProjectArea();
		if (projectArea &&
			projectArea.itemId) {
			return projectArea.itemId;
		} 
		return null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 * 				String name
 	 * 				String itemId
 	 */	
	_getProjectArea: function() {
		// Report via saved query
		var scopeItem = this.getScopeItem();
		if (scopeItem) {
			return scopeItem;
		}
		// Report via primed values
		// Report via default values
		if (this._local) {
			// Local viewlet
			return com.ibm.team.process.web.projectArea;
		}
		// Remote viewlet
		return null;
	},

	
	_stopEditing: function(saving){
		if(!saving){
			return;
		}
		if(this._height && this._height != null){
			this.getViewletState().set("height", this._height);
		}
		if(this._width && this._width != null){
			this.getViewletState().set("width", this._width);
		}
	},
	
	_resetCookieFunc: function() {
		if(this._customReportingUrl == -1 || this._customReportingUrl == undefined){
			setTimeout(dojo.hitch(this, "_resetCookieFunc"), COOKIE_RESET_INTERVAL);
			return;
		}
		jazz.client.xhrGet
		({
			url: this._customReportingUrl + "?b_action=xts.run&m=portal/close.xts",
			headers: this.headers,
			preventCache: false,
			handleAs: "text"
		});
		setTimeout(dojo.hitch(this, "_resetCookieFunc"), COOKIE_RESET_INTERVAL);
	}
		
});

})();
