/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007, 2011,
 * 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.EtlConfig");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.util.html");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.reports.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.reports.web.ui.internal.ValidatingTextBox");

dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal",
		"ReportsManagementMessages");

(function() { // module container
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var RemoteSnapshotTable = null;
	var LocalSnapshotTable = null;
	dojo
			.declare(
					"com.ibm.team.reports.web.ui.internal.EtlConfig",
					[ dijit._Widget, dijit._Templated ],
					{
						/* src to icon */
						__remoteUpdateAllIconSrc : null,
						__startRemoteScheduledTaskSrc : null,
						__localUpdateAllIconSrc : null,

						templatePath : dojo.moduleUrl(
								"com.ibm.team.reports.web",
								"ui/internal/templates/EtlConfig.html"),

						constructor : function() {
							this.messages = dojo.i18n.getLocalization(
									"com.ibm.team.reports.web.ui.internal",
									"ReportsManagementMessages");
							this.__remoteUpdateAllIconSrc = dojo
									.moduleUrl("com.ibm.team.reports.web",
											"ui/internal/images/run-all-etl-jobs_co.gif");
							this.__localUpdateAllIconSrc = dojo
									.moduleUrl("com.ibm.team.reports.web",
											"ui/internal/images/run-all-etl-jobs_co.gif");
							this.__startRemoteScheduledTaskSrc = dojo
									.moduleUrl("com.ibm.team.reports.web",
											"ui/internal/images/run-all-etl-jobs_co.gif");

						},

						postCreate : function() {
							com.ibm.team.reports.web.ui.internal.EtlConfig.superclass.postCreate
									.apply(this);
							this.header = new jazz.ui.MessageBox();
							this.headerArea.appendChild(this.header.domNode);
							this._saveAllButton.disabled = true;
							dojo.connect(this._saveAllButton, "onclick", this,
									"_doSave");
							this.connect(this._remoteUpdateAllLink, "onclick",
									this._handleUpdateAllRemote);
							this.connect(this._localUpdateAllLink, "onclick",
									this._handleUpdateAllLocal);
							this.connect(this._startRemoteScheduledTaskLink,
									"onclick", this._handleStartRemoteTask);
							this._createProviders();
							this._fetchProviders();
							this._fetchShouldHideLegacy();
							this.connect(this.updateAll, "onsubmit",
									this._updateAllHandler);

							this._fetchXdcAuthType();
							
							this._checkIsJTS();
							this._fetchSnapshotDescriptors();

							this._remoteSnapshots = new Array();
							this._dirtyRemoteSnapshots = new Array();
							this._remoteSnapshotNames = new Array();

						},

						_checkIsJTS : function() {
							var srh = new ServiceResponseHandler(this,
									"_handleJTSCheck", "_handleJTSCheckFailure");
							ServerStatusClient.getIsJazzTeamServer(srh);
						},

						_handleJTSCheck : function(response) {
							this.isJTS = response == "true";
							if (!this.isJTS) {
								this._checkIsStandalone();
							}
							this._createJTSSpecificControls();
						},

						_checkIsStandalone : function() {
							var srh = new ServiceResponseHandler(this,
									"_handleStandaloneCheck",
									"_handleJTSCheckFailure");
							ServerStatusClient.getServerModeString(srh);
						},

						_handleStandaloneCheck : function(response) {
							this.isJTS = response.toLowerCase() === "standalone";
							this._createJTSSpecificControls();
						},

						_handleJTSCheckFailure : function(errorObj) {
							this.isJTS = true;
							this._createJTSSpecificControls();
						},

						_createJTSSpecificControls : function() {
							while(this._etlDescription.hasChildNodes()){
								this._etlDescription.removeChild(this._etlDescription.firstChild);
							}
							this._etlDescription.appendChild(document.createTextNode(this.messages.etlDescription1));
							if (this.isJTS) {
								dojo.removeClass(this._schedulingSection,
										"hidden");
								this._createScheduling();
								this._fetchScheduling();
								this._etlDescription.appendChild(document.createTextNode(this.messages.etlDescription2));
								dojo.removeClass(this.RemoteScheduledTaskDiv, "hidden");
							} else {
								dojo
										.addClass(this._schedulingSection,
												"hidden");
								dojo.addClass(this.RemoteScheduledTaskDiv, "hidden");
							}
							this._etlDescription.appendChild(document.createTextNode(this.messages.etlDescription3));
						},

						_createProviders : function() {
							this.RemoteRadio = document.createElement("input");
							this.RemoteRadio.type = "radio";
							this.RemoteRadio.name = "providerGroup";
							this.RemoteRadio.value = "Remote";
							this.RemoteRadio.id = "remoteRadioId";
							this.RemoteRadio.style.margin = "5px";
							var remoteRadioLabel = document
									.createElement("label");
							remoteRadioLabel.style.margin = "5px";
							remoteRadioLabel.appendChild(document
									.createTextNode(this.messages.remoteRadio));
							remoteRadioLabel.htmlFor = this.RemoteRadio.id;

							this.LocalRadio = document.createElement("input");
							this.LocalRadio.type = "radio";
							this.LocalRadio.name = "providerGroup";
							this.LocalRadio.value = "Local";
							this.LocalRadio.id = "localRadioId";
							this.LocalRadio.style.margin = "5px";
							var localRadioLabel = document
									.createElement("label");
							localRadioLabel.style.margin = "5px";
							localRadioLabel.appendChild(document
									.createTextNode(this.messages.localRadio));
							localRadioLabel.htmlFor = this.LocalRadio.id;

							this._providerForm.appendChild(this.RemoteRadio);
							this._providerForm.appendChild(remoteRadioLabel);
							this._providerForm.appendChild(document
									.createElement("br"));
							this._providerForm.appendChild(this.LocalRadio);
							this._providerForm.appendChild(localRadioLabel);
							dojo.connect(this.LocalRadio, "onclick", this,
									function(e) {
										this._updateDirty();
									});
							dojo.connect(this.RemoteRadio, "onclick", this,
									function(e) {
										this._updateDirty();
									});
						},

						_fetchProviders : function() {
							var providerHandler = new ServiceResponseHandler(
									this, "_fetchProvidersHandler",
									"_remoteError");

							ReportsClient
									.getDataWarehouseProvider(providerHandler);
						},

						_fetchXdcAuthType : function() {
							var authHandler = new ServiceResponseHandler(
									this, "_fetchXdcAuthTypeHandler",
									"_remoteError");

							ReportsClient
									.getXdcAuthType(authHandler);
						},
						
						_fetchProvidersHandler : function(provider) {
							this.SavedProvider = provider;
							if (this.RemoteRadio.value.toLowerCase() == provider
									.toLowerCase()) {
								this.RemoteRadio.checked = true;
							} else {
								this.LocalRadio.checked = true;
							}
							this._updateSchedulingEnabled();
						},
						
						_fetchXdcAuthTypeHandler : function(authType) {
							this.SavedAuthType = authType;
							this._createToolbars();
						},
						
						_fetchShouldHideLegacy : function() {
							var providerHandler = new ServiceResponseHandler(
									this, "_fetchShouldHideLegacyHandler",
									"_noop");//not a big deal if this fails
							ReportsClient
									.getShouldHideLegacy(providerHandler);
						},

						_fetchShouldHideLegacyHandler : function(hide) {
							if(hide && hide == true){
								dojo.addClass(this._localSnapshotSection, "hidden");
								this.LocalRadio.disabled = hide;
							}
						},
						
						_updateSchedulingEnabled : function() {
							if (this.isJTS && this.SavedProvider) {
								this.DelayInput.disabled = this.SavedProvider
										.toLowerCase() === this.LocalRadio.value
										.toLowerCase();
							}
						},

						_fetchSnapshotDescriptors : function() {
							var remoteHandler = new ServiceResponseHandler(
									this, "_fillRemoteSnapshotList",
									"_remoteError");
							var localHandler = new ServiceResponseHandler(this,
									"_fillLocalSnapshotList", "_remoteError");
							ReportsClient.getSnapshotDescriptors(remoteHandler,
									{
										flags : ReportsClient.REMOTE
									});
							ReportsClient.getSnapshotDescriptors(localHandler,
									{
										flags : ReportsClient.LOCAL
									});
						},

						_createToolbars : function() {
							this._createSnapshotTables();
						},

						_createScheduling : function() {
							if (!this.isJTS) {
								return;
							}
							this.StartTime = new com.ibm.team.reports.web.ui.internal.ValidatingTextBox();
							this.StartTime
									.setErrorMessage(this.messages.startTimeValidationError);
							this.StartTime.validate = function(value) {
								return value - 0 == value && value.length > 0
										&& value <= 24 && value >= 1;
							};
							this.StartTimeInput = this.StartTime.inputNode;
							dojo.connect(this.StartTimeInput, "onkeyup", this,
									function(e) {
										this._updateDirty();
									});
							dojo.addClass(this.StartTimeInput, "smallTextBox");
							this.StartTimeDiv
									.appendChild(this.StartTime.domNode);

							this.Delay = new com.ibm.team.reports.web.ui.internal.ValidatingTextBox();
							this.Delay
									.setErrorMessage(this.messages.delayValidationError);
							this.Delay.validate = function(value) {
								return value - 0 == value && value.length > 0 && value > 0;
							};
							this.DelayInput = this.Delay.inputNode;
							this.DelayInput.disabled = true;
							dojo.connect(this.DelayInput, "onkeyup", this,
									function(e) {
										this._updateDirty();
									});
							dojo.addClass(this.DelayInput, "smallTextBox");
							this.DelayDiv.appendChild(this.Delay.domNode);
						},

						_fetchScheduling : function() {
							var handler = new ServiceResponseHandler(this,
									"_fetchSchedulingHandler", "_remoteError");
							ReportsClient
									.getSnapshotSchedulingDefaults(handler);
							this._updateSchedulingEnabled();
						},

						_fetchSchedulingHandler : function(defaults) {
							this.StartTimeInput.value = defaults[0] ? defaults[0]
									: "";
							this.DelayInput.value = defaults[1] ? defaults[1] / 3600
									: "";
							this.SavedStartTime = this.StartTimeInput.value;
							this.SavedDelay = this.DelayInput.value;
							this._updateDirty();
						},

						_doSave : function() {
							if (!this._isValid()) {
								this._displayErrorMessage(
										this.messages.invalidSaveValues, true);
								return;
							}
							var argArray = new Array();
							var providerValue = this.RemoteRadio.checked ? this.RemoteRadio.value
									: this.LocalRadio.value;
							if(providerValue && this.SavedProvider && providerValue.toLowerCase() != this.SavedProvider.toLowerCase()){
								argArray
										.push(this
												._getPropertyArguments(
														"datawarehouse.provider",
														providerValue,
														"com.ibm.team.datawarehouse.service.internal.InternalDataWarehouseService"));
							}
							if (this.isJTS) {
								var delay = this.DelayInput.value ? (this.DelayInput.value * 3600)
										.toString()
										: "";
								var startTime = this.StartTimeInput.value ? this.StartTimeInput.value
										: "";
								if(startTime && this.SavedStartTime && startTime != this.SavedStartTime){
									argArray
											.push(this
													._getPropertyArguments(
															"snapshot.time",
															startTime,
															"com.ibm.team.datawarehouse.service.internal.InternalDataWarehouseService"));
								}
								if(delay && this.SavedDelay && delay != this.SavedDelay * 3600){
									argArray
											.push(this
													._getPropertyArguments(
															"RemoteSnapshotManagerTask.fixedDelay",
															delay,
															"com.ibm.team.datawarehouse.service.internal.RemoteSnapshotManagerTask"));
								}
							}
							for ( var snapshot in this._dirtyRemoteSnapshots)
								if (this._dirtyRemoteSnapshots
										.hasOwnProperty(snapshot)) {
									var dirty = this._dirtyRemoteSnapshots[snapshot];
									var original = this._remoteSnapshots[dirty.name];
									var userid = this
											._getSnapshotPropertyArguments(
													dirty, original, "userId",
													"userid");
									if (userid) {
										argArray.push(userid);
									}
									var password = this
											._getSnapshotPropertyArguments(
													dirty, original,
													"password", "password");
									if (password) {
										argArray.push(password);
									}
									var enabled = this
											._getSnapshotPropertyArguments(
													dirty, original, "enabled",
													"enabled");
									if (enabled) {
										argArray.push(enabled);
									}
								}

							var args = {
								'updates' : dojo.toJson(argArray)
							};
							var srh = new ServiceResponseHandler(this,
									"_saveHandler", "_remoteError");
							ServerStatusClient.postConfigurationUpdates(srh,
									args);
						},

						_saveHandler : function() {
							this.SavedProvider = this.RemoteRadio.checked ? this.RemoteRadio.value
									: this.LocalRadio.value;
							if (this.isJTS) {
								this.SavedStartTime = this.StartTimeInput.value;
								this.SavedDelay = this.DelayInput.value;
								this._updateSchedulingEnabled();
							}
							this._updateDirty();
							this._fetchSnapshotDescriptors();
							if (this.RemoteRadio.checked) {
								dojo.removeClass(this.RemoteScheduledTaskDiv,
										"hidden");
							} else {
								dojo.addClass(this.RemoteScheduledTaskDiv,
										"hidden");
							}
							this
									._displaySuccessMessage(this.messages.saveSuccessful);
						},

						_isValid : function() {
							if (this.isJTS) {
								return this.Delay.isValid()
										&& this.StartTime.isValid();
							} else {
								return true;
							}
						},

						_updateDirtySnapshots : function() {
							for ( var snapshot in this._dirtyRemoteSnapshots)
								if (this._dirtyRemoteSnapshots
										.hasOwnProperty(snapshot)) {
									this._dirtyRemoteSnapshots[snapshot].password = '';
									if (this._dirtyRemoteSnapshots[snapshot].passwordInput) {
										this._dirtyRemoteSnapshots[snapshot].passwordInput.value = '';
									}
									this._remoteSnapshots[snapshot] = dojo
											.clone(this._dirtyRemoteSnapshots[snapshot]);
								}

							this._updateDirty();
						},

						_getSnapshotPropertyArguments : function(dirty,
								original, property, propertyName) {
							if (dirty[property] == original[property]) {
								return null;
							}
							var args = {
								'name' : dirty.name + ".remoteSnapshot."
										+ propertyName,
								'value' : dirty[property].toString(),
								'serviceImplementationName' : dirty.qualifiedClassName
							};
							return this._getPropertyArguments(dirty.name
									+ ".remoteSnapshot." + propertyName,
									dirty[property].toString(),
									dirty.qualifiedClassName);
						},

						_getPropertyArguments : function(name, value,
								implementationName) {
							var args = {
								'name' : name,
								'value' : value,
								'serviceImplementationName' : implementationName
							}
							return args;
						},

						_createSnapshotTables : function() {
							RemoteSnapshotTable = new jazz.ui.StyledTable( {
								columns : [ {
									label : this.messages.enabled,
									sortable : false,
									width : "5%"
								}, {
									label : this.messages.dataCollectionJob,
									sortable : false,
									width : "300px"
								}, {
									label : this.messages.state,
									sortable : false,
									width : "7%"
								}, {
									label : (this.SavedAuthType == "jts")?this.messages.consumerKey:this.messages.userId,
									sortable : false,
									width : "15%"
								}, {
									label : (this.SavedAuthType == "jts")?this.messages.consumerSecret:this.messages.password,
									sortable : false,
									width : "15%"
								}, {
									label : this.messages.actions,
									sortable : false,
									width : "5%"
								} ],
								sortBy : 2,
								sortByAscending : false,
								handleSort : function() {
								},
								trim : jazz.ui.styledTable.TrimStyles.BLUE
							});

							LocalSnapshotTable = new jazz.ui.StyledTable( {
								columns : [ {
									label : this.messages.dataCollectionJob,
									sortable : false
								}, {
									label : this.messages.actions,
									sortable : false,
									width : "5%"
								} ],
								sortBy : 2,
								sortByAscending : false,
								handleSort : function() {
								},
								trim : jazz.ui.styledTable.TrimStyles.BLUE
							});

							RemoteSnapshotTable.domNode.style.width = "100%";
							LocalSnapshotTable.domNode.style.width = "100%";

							this.remoteSnapshotList
									.appendChild(RemoteSnapshotTable.domNode);
							this.localSnapshotList
									.appendChild(LocalSnapshotTable.domNode);
						},

						_handleUpdateAllRemote : function(e) {
							e.preventDefault();
							if(!this._saveAllButton.disabled){
								this._displayMessage(this.messages.savePageBeforeRunning,
									jazz.ui.MessageBox.STATUS_TYPES.ERROR,
									false);
								return;
							}
							this._updateSnapshots(e, {
								flags : ReportsClient.REMOTE
							});
						},

						_handleUpdateAllLocal : function(e) {
							e.preventDefault();
							this._updateSnapshots(e, {
								flags : ReportsClient.LOCAL
							});
						},

						_handleStartRemoteTask : function(e) {
							e.preventDefault();
							if(!this._saveAllButton.disabled){
								this._displayMessage(this.messages.savePageBeforeRunning,
									jazz.ui.MessageBox.STATUS_TYPES.ERROR,
									false);
								return;
							}
							var handler = new ServiceResponseHandler(this,
									"_noop", "_noop");
							ReportsClient
									.postUpdateAllRemoteSnapshotData(handler);
							this._displayInfoMessage(
									this.messages.snapshotStarted, true);
						},

						_updateSnapshots : function(e, args) {
							e.preventDefault();
							var handler = new ServiceResponseHandler(this,
									"_noop", "_noop");
							ReportsClient.postUpdateSomeSnapshotData(handler,
									args);
							if (this.RemoteRadio.checked) {
								this._displayInfoMessage(
										this.messages.snapshotStarted, true);
							} else {
								this._displayInfoMessage(
										this.messages.snapshotStartedDataMart, true);
							}
						},

						refresh : function() {
							this.header.hideMessage();
						},

						_noop : function() {
						},

						_displayErrorMessage : function(message, closeable) {
							this._displayMessage(message,
									jazz.ui.MessageBox.STATUS_TYPES.ERROR,
									closeable);
						},

						_displaySuccessMessage : function(message, closeable) {
							this._displayMessage(message,
									jazz.ui.MessageBox.STATUS_TYPES.OK,
									closeable);
						},

						_displayInfoMessage : function(message, closeable) {
							this._displayMessage(message,
									jazz.ui.MessageBox.STATUS_TYPES.INFO,
									closeable);
						},

						_displayMessage : function(message, messageType,
								closeable) {
							if (!closeable) {
								closeable = false;
							}
							if (this.header) {
								var status = {
									type : messageType,
									summary : message,
									closeable : closeable
								};
								this.header.showMessage(status);
							}
						},

						_removeChildren : function(element) {
							while (element.firstChild) {
								element.removeChild(element.firstChild);
							}
						},

						_remoteError : function() {
							this
									._displayErrorMessage(this.messages.errorInResponse);
						},

						_updateDirty : function() {
							var dirty = false;
							if (this.RemoteRadio.checked && this.SavedProvider) {
								dirty = this.SavedProvider.toLowerCase() != this.RemoteRadio.value
										.toLowerCase();
							}
							if (this.LocalRadio.checked && this.SavedProvider) {
								dirty = this.SavedProvider.toLowerCase() != this.LocalRadio.value
										.toLowerCase();
							}
							if (this.isJTS && this.DelayInput
									&& this.SavedDelay != this.DelayInput.value) {
								dirty = true;
							}

							if (this.isJTS
									&& this.StartTimeInput
									&& this.SavedStartTime != this.StartTimeInput.value) {
								dirty = true;
							}

							for ( var snapshot in this._remoteSnapshotNames)
								if (this._remoteSnapshotNames
										.hasOwnProperty(snapshot)) {
									var name = this._remoteSnapshotNames[snapshot];
									var dirtySnapshot = this._dirtyRemoteSnapshots[name];
									var original = this._remoteSnapshots[name];

									if (dirtySnapshot) {
										if (dirtySnapshot.userId == original.userId
												&& dirtySnapshot.password == original.password
												&& dirtySnapshot.enabled == original.enabled) {
										} else {
											dirty = true;
										}
									}
								}
							this._saveAllButton.disabled = !dirty;
						},

						_fillRemoteSnapshotList : function(items) {
							while (RemoteSnapshotTable.tbody.hasChildNodes()) {
								RemoteSnapshotTable.tbody
										.removeChild(RemoteSnapshotTable.tbody.firstChild);
							}
							if (items === null) {
								return;
							}
							var toolbarRunArgs = {};
							toolbarRunArgs.label = this.messages.runSnapshot;
							toolbarRunArgs.imgSrc = dojo.moduleUrl(
									"com.ibm.team.reports.web",
									"ui/internal/images/run-etl-job_co.gif");
							
							var toolbarRunFullArgs = {};
							toolbarRunFullArgs.label = this.messages.runFullSnapshot;
							toolbarRunFullArgs.imgSrc = dojo.moduleUrl(
									"com.ibm.team.reports.web",
									"ui/internal/images/run-full-etl-job_co.gif");

							var toolbarPingArgs = {};
							toolbarPingArgs.label = this.messages.testSnapshot;
							toolbarPingArgs.imgSrc = dojo
									.moduleUrl("com.ibm.team.reports.web",
											"ui/internal/images/test-data-connection_co.gif");

							for ( var i = 0; i < items.length; i++) {
								var item = items[i];
								item.password = item.password ? item.password
										: "********";
								this._remoteSnapshots[item.name] = dojo
										.clone(item);
								this._dirtyRemoteSnapshots[item.name] = dojo
										.clone(item);
								this._remoteSnapshotNames.push(item.name);
								var row, enabled, name, start, userId, password, actions, state;
								row = document.createElement("tr");

								enabled = document.createElement("td");
								name = document.createElement("td");
								userId = document.createElement("td");
								password = document.createElement("td");
								actions = document.createElement("td");
								state = document.createElement("td");

								dojo.addClass(enabled, "center-column");
								var enabledInput = document.createElement("input");
								enabledInput.type = "checkbox";
								enabled.appendChild(enabledInput);
								enabledInput.checked = item.enabled;
								enabledInput.item = item;
								enabledInput.disabled = item.currentlyRunning == true;
								dojo
										.connect(
												enabledInput,
												"onclick",
												this,
												function(e) {
													var target = this
															._getCurrentTarget(e);
													this._dirtyRemoteSnapshots[target.item.name].enabled = target.checked;
													this._updateDirty();
												});
								

								name.innerHTML = jazz.util.html
										.escape(item.displayName);
								state.innerHTML = item.currentlyRunning == true ? this.messages.running : this.messages.idle;
								
								if (item.configurable == true) {
									var userIdInput = document
											.createElement("input");
									userIdInput.value = item.userId;
									userIdInput.disabled = item.currentlyRunning == true;
									userIdInput.item = item;
									dojo
											.connect(
													userIdInput,
													"onkeyup",
													this,
													function(e) {
														var target = this
																._getCurrentTarget(e);
														this._dirtyRemoteSnapshots[target.item.name].userId = target.value;
														this._updateDirty();
													});
									userId.appendChild(userIdInput);
									

									var passwordInput = document
											.createElement("input");
									passwordInput.disabled = item.currentlyRunning == true;
									passwordInput.type = "password";
									passwordInput.value = item.password ? item.password
											: "********";
									passwordInput.item = item;
									dojo
											.connect(
													passwordInput,
													"onkeyup",
													this,
													function(e) {
														var target = this
																._getCurrentTarget(e);
														this._dirtyRemoteSnapshots[target.item.name].password = target.value;
														this._dirtyRemoteSnapshots[target.item.name].passwordInput = target;
														this._updateDirty();
													});
									password.appendChild(passwordInput);
								}
								var toolbar = new com.ibm.team.reports.web.ui.internal.common.Toolbar();

								toolbarRunArgs.callbackData = {
									snapshotName : item.name,
									enabledInput : enabledInput,
									userIdInput : userIdInput,
									passwordInput : passwordInput,
									stateLabel : state
								};
								
								toolbarRunFullArgs.callbackData = {
										snapshotName : item.name,
										enabledInput : enabledInput,
										userIdInput : userIdInput,
										passwordInput : passwordInput,
										stateLabel : state
									};

								var callback = dojo
										.hitch(
												this,
												function(e) {
													if(this._saveAllButton.disabled){
														this
																._runSnapshotHandler(e.currentTarget.callbackData.snapshotName, true, true);
														e.currentTarget.callbackData.enabledInput.disabled = true;
														e.currentTarget.callbackData.userIdInput.disabled = true;
														e.currentTarget.callbackData.passwordInput.disabled = true;
														e.currentTarget.callbackData.stateLabel.innerHTML = this.messages.running;
													}else{
														this._displayMessage(this.messages.savePageBeforeRunning,
															jazz.ui.MessageBox.STATUS_TYPES.ERROR, false);
													}
												});

								toolbar.addButton(this, callback,
										toolbarRunArgs);
								var callback = dojo
										.hitch(
												this,
												function(e) {
													if(this._saveAllButton.disabled){
														if(confirm(this.messages.confirmRunFullJob + "\n\n" + this.messages.confirmRunFullJob2 + "\n\n" + this.messages.confirmRunFullJob3)){
															this
																	._runSnapshotHandler(e.currentTarget.callbackData.snapshotName, false, true);
															e.currentTarget.callbackData.enabledInput.disabled = true;
															e.currentTarget.callbackData.userIdInput.disabled = true;
															e.currentTarget.callbackData.passwordInput.disabled = true;
															e.currentTarget.callbackData.stateLabel.innerHTML = this.messages.running;
														}
													}else{
														this._displayMessage(this.messages.savePageBeforeRunning,
															jazz.ui.MessageBox.STATUS_TYPES.ERROR, false);
													}
												});

								toolbar.addButton(this, callback,
										toolbarRunFullArgs);

								dojo.addClass(toolbar.domNode, "table-actions");
								toolbar.domNode.style.visibility = "hidden";
								row.toolbar = toolbar;
								row.onmouseover = dojo
										.hitch(
												this,
												function(e) {
													var target = this
															._getCurrentTarget(e);
													if (dojo.isIE) {
														while (target.parentNode
																&& !target.toolbar) {
															target = target.parentNode;
														}
													}
													target.toolbar.domNode.style.visibility = "visible";
												});
								row.onmouseout = dojo
										.hitch(
												this,
												function(e) {
													var target = this
															._getCurrentTarget(e);
													if (dojo.isIE) {
														while (target.parentNode
																&& !target.toolbar) {
															target = target.parentNode;
														}
													}
													target.toolbar.domNode.style.visibility = "hidden";
												});
								actions.appendChild(toolbar.domNode);

								row.appendChild(enabled);
								row.appendChild(name);
								row.appendChild(state);
								row.appendChild(userId);
								row.appendChild(password);
								row.appendChild(actions);

								RemoteSnapshotTable.tbody.appendChild(row);

								this._updateDirtySnapshots();
							}

						},

						_fillLocalSnapshotList : function(items) {
							while (LocalSnapshotTable.tbody.hasChildNodes()) {
								LocalSnapshotTable.tbody
										.removeChild(LocalSnapshotTable.tbody.firstChild);
							}
							if (items !== null) {
								var toolbarRunArgs = {};
								toolbarRunArgs.label = this.messages.runSnapshot;
								toolbarRunArgs.imgSrc = dojo
										.moduleUrl("com.ibm.team.reports.web",
												"ui/internal/images/run-etl-job_co.gif");

								for ( var i = 0; i < items.length; i++) {
									var item = items[i];
									var row, name, actions;

									row = document.createElement("tr");
									name = document.createElement("td");
									actions = document.createElement("td");

									name.innerHTML = jazz.util.html
											.escape(item.displayName);

									var toolbar = new com.ibm.team.reports.web.ui.internal.common.Toolbar();

									toolbarRunArgs.callbackData = {
										snapshotName : item.name
									};

									var callback = dojo
											.hitch(
													this,
													function(e) {
														this
																._runSnapshotHandler(e.currentTarget.callbackData.snapshotName);
													});

									toolbar.addButton(this, callback,
											toolbarRunArgs);

									dojo.addClass(toolbar.domNode,
											"table-actions");
									toolbar.domNode.style.visibility = "hidden";
									row.toolbar = toolbar;
									row.onmouseover = dojo
											.hitch(
													this,
													function(e) {
														var target = this
																._getCurrentTarget(e);
														if (dojo.isIE) {
															while (target.parentNode
																	&& !target.toolbar) {
																target = target.parentNode;
															}
														}
														target.toolbar.domNode.style.visibility = "visible";
													});
									row.onmouseout = dojo
											.hitch(
													this,
													function(e) {
														var target = this
																._getCurrentTarget(e);
														if (dojo.isIE) {
															while (target.parentNode
																	&& !target.toolbar) {
																target = target.parentNode;
															}
														}
														target.toolbar.domNode.style.visibility = "hidden";
													});

									actions.appendChild(toolbar.domNode);

									row.appendChild(name);
									row.appendChild(actions);

									LocalSnapshotTable.tbody.appendChild(row);
								}
							}
						},

						_getCurrentTarget : function(e) {
							if (dojo.isIE) {
								return window.event.srcElement;
							}
							if (e) {
								return e.currentTarget;
							}
							return undefined;
						},

						_runSnapshotHandler : function(name, delta, overrideDisabled) {
							if(!overrideDisabled){
								overrideDisabled = false;
							}
							var handler = new ServiceResponseHandler(this,
									"_noop", "_noop");
							ReportsClient.postUpdateSnapshotData(handler, {
								snapshotName : name,
								delta : delta,
								overrideDisabled : overrideDisabled
							});
							if (this.RemoteRadio.checked) {
								this._displayInfoMessage(
										this.messages.snapshotStarted, true);
							} else {
								this._displayInfoMessage(
										this.messages.snapshotStartedDataMart, true);
							}
						},

						_pingSnapshot : function(name) {
							var handler = new ServiceResponseHandler(this,
									"_pingSnapshotHandler",
									"_pingSnapshotHandler");

							ReportsClient.postPingRemoteSnapshot(handler, {
								snapshotName : name
							});
						},

						_pingSnapshotHandler : function(success) {
							if (success === "true") {
								this._displaySuccessMessage(
										this.messages.pingSuccess, true);
							} else {
								this._displayErrorMessage(
										this.messages.pingFailed, true);
							}
						}
					});

})();
