/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.NewReportEditor");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.IconItem");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var session = com.ibm.team.repository.web.client.session;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

var SHARED = 1 << 0;
var PRIVATE = 1 << 2;
var SYSTEM = 1 << 3;

/**
 * Widget to create a new report query
 */
var NewReportEditor = dojo.declare("com.ibm.team.reports.web.ui.internal.NewReportEditor", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/NewReportEditor.html"),
		_csPageHelpWidgets: null,
		csHelpNode: null,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			this._csPageHelpWidgets= [];
		},
				
		postCreate: function() {
			this._reportUUID = this.reportUUID;
			this._responseCount = 0;
			this.typeIcon.src = ImagePool.REPORT_OBJ;
			this._allFolders = [];
			// A map mapping folder nodes to the underlying folder
			this._folderMap = {};
			// A map mapping template nodes to the underlying template uuid
			this._templateMap = {};
			
			this._createFolderTree();
			this._createTemplateTree();
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			var responseHandler = new ServiceResponseHandler(
					this,
					"_addTemplates",
					"_remoteError");
			ReportsClient.getReportDescriptors(responseHandler, {flags: ReportsClient.EXCLUDE_MICRO});
			
			var args = {
				ownerUUID: session.getAuthenticatedContributor().itemId,
				flags: PRIVATE
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
			args = {
				flags: (SHARED | SYSTEM)
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
			this._createContextSensitiveHelp();
		},
		
		_createContextSensitiveHelp: function() {
			dojo.empty(this.csHelpNode);
			this._createCSHWidget(NewReportEditor.CSH_NEW_REPORT_EDITOR);
		},
	
		_createCSHWidget: function(uaKey) {
			var parms = {};
			parms.uaKey = uaKey;
			var pageHelp = new PageHoverHelp(parms);
			this.csHelpNode.appendChild(pageHelp.domNode);
			this._csPageHelpWidgets.push(pageHelp);
		},
		
		_handleSave: function() {
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			var currentTemplateNode = this._templateTreeWidget.getSelection();
			var templateUUID = this._templateMap[currentTemplateNode];
			if (!templateUUID || templateUUID == null) {
				this._showErrorMessage(this.messages.pleaseSelectTemplate);
				return;
			}
			var args = {
		        name: this._getName(),
		        description: this._getDescription(),
		        reportUUID: templateUUID
	        }
	        var currentFolderNode = this._folderTreeWidget.getSelection();
	        var currentFolder = this._folderMap[currentFolderNode];
	        if (!currentFolder || currentFolder == null) {
	        	this._showErrorMessage(this.messages.pleaseSelectFolder);
	        	return;
	        }
	        
	        // If this is a special folder, just set the sharing flag. Otherwise
	        // set the parent folder.
	        if (currentFolder.processAreaUUID == null) {
	        	args.sharedIfNullFolder = currentFolder.shared;
	        } else {
	        	args.folderUUID = currentFolder.folderUUID;
	        }
	        var responseHandler = new ServiceResponseHandler(
					this,
					"_saveComplete",
					"_remoteError");
			ReportsClient.postSaveQueryDescriptor(responseHandler, args);
		},
		
		_handleCancel: function() {
			history.go(-1);
		},
		
		_getName: function() {
			return this._nameInput.value;
		},
		
		_getDescription: function() {
			return this._descriptionText.value;
		},
		
		_saveComplete: function(args) {
			document.location.href = "#action=com.ibm.team.reports.viewQuery&queryUUID=" + args;
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this._showErrorMessage("Error invoking service; message: " + error.message);
		},

		_addTemplates: function(args) {
			if (args == null) args = [];
			// First sort them alphabetically
			function sortFunction(a, b) {
				return a.name < b.name ? -1 : (a.name > b.name ? 1 : 0);
			}
			var sorted = args.sort(sortFunction);
			this._templates = sorted;
			
			// Create the tree and add the items
			var itemToSelect = null;
			for (var i = 0; i < this._templates.length; i++) {
				var item = new jazz.ui.tree.IconItem({label: this._templates[i].name, iconClass: "iconTemplate"});
				this._templateMap[item] = this._templates[i].reportUUID;
				// If we aren't pre-selecting anything, and this is the first item, select it.
				if (i == 0 && !this._reportUUID) {
					itemToSelect = item;
				}
				this._templateTreeWidget.addChild(item);
				if (this._reportUUID && (this._templates[i].reportUUID == this._reportUUID)) {
					itemToSelect = item;
				}
			}
			
			if (itemToSelect != null) {
				this._templateTreeWidget.setSelection(itemToSelect);
			}
		},
		
		_addFolders: function(folders) {
			this._responseCount++;
			if (!folders) folders = [];
			for (var i = 0; i < folders.length; i++) {
				this._allFolders.push(folders[i]);
			}
			if (this._responseCount == 1) {
				return;
			}
			this._addFoldersToTree();
		},
		
		_addFoldersToTree: function() {
			// First, find the two root folders
			var sharedRoot = null;
			var privateRoot = null;
			for (var i = 0; i < this._allFolders.length; i++) {
				var folder = this._allFolders[i];
				if (folder.system) {
					if (folder.shared) {
						sharedRoot = folder;
					} else {
						privateRoot = folder;
					}
				}
			}

			if (privateRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: privateRoot.name, iconClass: "iconFolder"});
				this._folderMap[rootItem] = privateRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, true, false);
			}
			if (sharedRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: sharedRoot.name, iconClass: "iconFolder"});
				this._folderMap[rootItem] = sharedRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, true, true);
			}
		},
		
		/**
		 * From the list of folders, find any that are children of parentFolder, and
		 * add them to the tree. Then recurse for each of the children.
		 */
		_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
			var foldersToAdd = [];
			for (var i = 0; i < folders.length; i++) {
				// Special case where the parent is a root system folder
				if (parentIsRoot) {
					if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name != parentFolder.name)) {
						foldersToAdd.push(folders[i]);
					}
				} else {
					if (folders[i].parentUUID == parentFolder.folderUUID) {
						foldersToAdd.push(folders[i]);
					}
				}
			}
			foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name > b.name;});
			for (var i = 0; i < foldersToAdd.length; i++) {
				var childNode = new jazz.ui.tree.IconItem({label: foldersToAdd[i].name, iconClass: "iconFolder"});
				this._folderMap[childNode] = foldersToAdd[i];
				parentFolderNode.addChild(childNode);
				this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
			}
		},

		_createFolderTree: function() {
			this._folderTreeWidget = new jazz.ui.Tree();
			this._folderTree.appendChild(this._folderTreeWidget.domNode);
		},
		
		_createTemplateTree: function() {
			this._templateTreeWidget = new jazz.ui.Tree();
			this._templateTree.appendChild(this._templateTreeWidget.domNode);
		},
		
		_handleNameChange: function() {
			this._updateEnablement();
		},
		
		_updateEnablement: function() {
			var name = this._getName();
			var hasName = (name && name.length > 0);
			this._saveButton.disabled = !hasName;
		},
		
		_showErrorMessage: function(message) {
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.showMessage(status);
		}
	}
);

NewReportEditor.CSH_NEW_REPORT_EDITOR = "com.ibm.jazz.reports.web.cshelp.new_report_editor";

})();