/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog");

dojo.require("com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");

var CognosReportsClient = com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var NewTemplateDialog = com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog;
var CognosReportsBrowseDialog = com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;

(function() { // module container

/**
 * Displays a dialog to upload a Cognos report to RTC server
 */
dojo.declare(
	"com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.cognos.web", "ui/internal/templates/NewTemplateDialog.html"),
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");

		// Set up folder list
        var foldersHandler = new ServiceResponseHandler(this, "_setFolders", "_FolderError");
        ReportsClient.getFolderDescriptors(foldersHandler, {flags: ReportsClient.SHARED});
				
		// Strings for the dialog
		this.labelOK = this.messages.labelOK;
		this.labelCancel = this.messages.labelCancel;
		this.labelFolder = this.messages.labelFolder;
		this.labelBrowse = this.messages.labelBrowse;
		this.labelProperties = this.messages.labelProperties;
		this.labelName = this.messages.labelName;
		this.labelDescription = this.messages.labelDescription;
		this.labelShared = this.messages.labelShared;
		this.labelReportDetails = this.messages.labelReportDetails;
		this.labelCognosReport = this.messages.labelCognosReport;
	},
	
	uninitialize: function() {
		if (this.cognosReportsBrowseDialog){
			this.cognosReportsBrowseDialog.destroy();
		}
	},
	
	_setFolders: function(folders) {
		console.debug(this.declaredClass + ":_setSharedFolders --> entry.");
		for (var i = 0; i < (folders ? folders.length : 0); i++) {
			var optn = document.createElement("OPTION");
			optn.text=folders[i].name;
			optn.value=folders[i].folderUUID;
			this._folderInput.options.add(optn);
		}
		var optn = document.createElement("OPTION");
		optn.text="System";
		optn.value="";
		this._folderInput.options.add(optn);
	},
	
	postCreate: function() {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.messages.labelImportReport,
			width: "390px",
			isClosable: true
		});
		this.connect(this._browseButton, "onclick", this._onBrowse);
		this.connect(this._okButton, "onclick", this._onOk);
		this.connect(this._cancelButton, "onclick", this._onCancel);
	},
	
	_onBrowse: function() {
	  var args = {
	  	viewer: this,
		elementsToBeDisplayed: "ReportTemplate"
	  }	;
	  var currentProjectArea = jazz.app.context.get();

	  this.cognosReportsBrowseDialog = new CognosReportsBrowseDialog(args);
	  this.connect(this.cognosReportsBrowseDialog._okButton, "onclick", this._onOkClick);	  
	},
	
	_onOkClick: function() {
		this.reportNode = this.cognosReportsBrowseDialog.node;
		this.reportPath = this.cognosReportsBrowseDialog.reportPath;
		this._IDInput.value = this.cognosReportsBrowseDialog.reportId;
		var fileName = this.reportNode.label;
		this._nameInput.value = fileName;
		this._reportInput.value = fileName;
		this._okButton.disabled = false;
		this.cognosReportsBrowseDialog.close();
	},
	
	_onCancel: function() {
		this.dialog.close();
	},
	
	_handleError: function(){
		alert(this.messages.errorCantCommunicate);
	},
	
	_Error: function(){
		alert(this.messages.errorCantCommunicate);
	},
	
	_FolderError: function(){
		alert(this.messages.errorCantCommunicate);
	},
	
	_onOk: function(){
		this.dialog.close();
	}
});

})();