/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.ui.internal.ProcessDescriptionViewlet"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.internal.PracticeElement"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.internal.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink");//$NON-NLS-1$
dojo.require("jazz.util.i18n"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.authoring.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
(function() {

	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants;
	var I18nUtil = jazz.util.i18n;
	var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient;
	var PracticeElement = com.ibm.team.process.authoring.web.common.internal.PracticeElement;
	var SERVICE_URI = "com.ibm.team.process.internal.service.web.IProcessWebUIService"; //$NON-NLS-1$
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
	var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
	var ResourceLink = jazz.ui.ResourceLink;
	var UIHelper = com.ibm.team.process.authoring.web.common.internal.UIHelper;
	
	dojo.declare("com.ibm.team.process.authoring.web.ui.internal.ProcessDescriptionViewlet", Viewlet, { //$NON-NLS-1$ 
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web", "ui/internal/templates/ProcessDescriptionViewlet.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.loading = this.messages.loading;
		},
		
		init: function() {
		},
		
		refresh: function() {
			delete this._message;
			// handle Auto(none) scope when no team areas
			if (!this.getScopeItem()) {
				// clear contents and show msg
				this._showMessage(this._createNoScopeMsg());
				this.getSite().doneRefresh();
				return;
			}
			
			this.getSite().setLoading(true);
			
			var processAreaType;
			if (this.getScope() == ProcessAuthoringConstants.PROJECT_AREA) {
				processAreaType = "project-area"; //$NON-NLS-1$
			} else if (this.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
				processAreaType = "team-area"; //$NON-NLS-1$
			} else {
				// clear contents and show msg
				this._showMessage(this._createNoScopeMsg());
				this.getSite().doneRefresh();
				return;
			}
			
			var self = this;
			ProcessAuthoringClient.checkProcessDescriptionForProcessArea({
				itemId: this.getScopeItem().itemId,
				processAreaType: processAreaType,
				load: function(data, ioArgs) {
					var descriptionURL = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
					ProcessAuthoringClient.getProcessDescription({
						resourceURL: descriptionURL,
						load: function(data, ioArgs) {
							self._descriptions = new PracticeElement({
								xml: data
							});
							self.update();
							self.getSite().setLoading(false);
							self.getSite().doneRefresh();
						},
						error: function(error) {
							self._handleError(error);
						}
					});
				},
				error: function(error) {
					var processAreaUrl;
					if (self.getScope() == ProcessAuthoringConstants.PROJECT_AREA) {
						processAreaUrl = UIHelper.getURLPrefix() + "/process/project-areas/" + self.getScopeItem().itemId; //$NON-NLS-1$
						self._getAssociatedProcessDescription(self, processAreaUrl);
					} else if (self.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
						var srh = new ServiceResponseHandler(self, "_populateProcessAreaData", "_handleError"); //$NON-NLS-1$ //$NON-NLS-2$
						var serviceRequest = new ServiceRequest(SERVICE_URI, "getTeamAreaByUUID", { //$NON-NLS-1$
							uuid: self.getScopeItem().itemId
						});
						TeamServerClient.invokeService(serviceRequest, srh);
						self.loadDoneHandle = dojo.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_LOADDONE, self, self._getAssociatedProcessDescription);
					} else {
						// clear contents and show msg
						self._showMessage(self._createNoScopeMsg());
						self.getSite().doneRefresh();
						return;
					}
				}
			});
		},
		
		_populateProcessAreaData: function(data) {
			if (data.processHierarchy && data.processHierarchy.itemId) {
				processAreaUrl = UIHelper.getURLPrefix() + "/process/project-areas/" + data.processHierarchy.itemId + "/team-areas/" + this.getScopeItem().itemId; //$NON-NLS-1$ //$NON-NLS-2$
				dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_LOADDONE, [this, processAreaUrl]);
			}
		},
		
		_getAssociatedProcessDescription: function(self, processAreaUrl) {
			ProcessAuthoringClient.checkAssociateSharedProcessDescriptionForProcessArea({
				targetUrl: processAreaUrl,
				load: function(data, ioArgs) {
					var associationURL = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
					ProcessAuthoringClient.getAssociation({
						url: associationURL,
						load: function(data, ioArgs) {
							var docElement = data.documentElement;
							var descriptionURL = null;
							var hasDescription = dojo.some(docElement.childNodes, function(node) {
								if (node.nodeType == ProcessAuthoringConstants.NodeType.ELEMENT) {
									if (UIHelper.isSpecifiedNode(node, "source", ProcessAuthoringConstants.NAMESPACE_JTP)) { //$NON-NLS-1$
										var sourceUrl = UIHelper.getAttributeNode(node, ProcessAuthoringConstants.XMLATTRIBUTE_RESOURCE, ProcessAuthoringConstants.NAMESPACE_RDF).value;
										if (sourceUrl !== null) {
											descriptionURL = sourceUrl;
											return true;
										} else {
											return false;
										}
									}
								}
							}, self);
							
							if (hasDescription) {
								ProcessAuthoringClient.getProcessDescription({
									resourceURL: descriptionURL,
									load: function(data, ioArgs) {
										self._descriptions = new PracticeElement({
											xml: data
										});
										self.update();
										self.getSite().setLoading(false);
										self.getSite().doneRefresh();
										self._unsubscribe();
									},
									error: function(error) {
										self._handleError(error);
										self._unsubscribe();
									}
								});
							}
						},
						
						error: function(error) {
							self._handleError(error);
							self._unsubscribe();
						}
					});
				},
				error: function(error) { //no associate shared process description
					self._handleError(error);
					self._unsubscribe();
				}
			});
			
		},
		
		_unsubscribe: function() {
			if (this.loadDoneHandle) {
				dojo.unsubscribe(this.loadDoneHandle);
				this.loadDoneHandle = null;
			}
		},
		
		_handleError: function(error) {
			if (error.status && error.status == ProcessAuthoringConstants.SC_FORBIDDEN) {
				var span = document.createElement("span"); //$NON-NLS-1$
				span.className = "errorText"; //$NON-NLS-1$
				span.appendChild(document.createTextNode(this.messages.permissionDenied));
				this._showMessage(span);
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
			} else if (error.status && error.status == ProcessAuthoringConstants.SC_NOT_FOUND) {
				this._summary.innerHTML = ""; //$NON-NLS-1$
				this._summary.className = "summary"; //$NON-NLS-1$
				this._details.innerHTML = ""; //$NON-NLS-1$
				this._details.className = ""; //$NON-NLS-1$
				if (this.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
					this._details.appendChild(document.createTextNode(this.messages.noDescTeam));
				} else {
					this._details.appendChild(document.createTextNode(this.messages.noDescProject));
				}
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
			} else {
				this.getSite().doneRefresh();
				this.getSite().error(error);
			}
		},
		
		update: function() {
			this._summary.innerHTML = ""; //$NON-NLS-1$
			this._summary.className = "summary"; //$NON-NLS-1$
			this._details.innerHTML = ""; //$NON-NLS-1$
			this._details.className = ""; //$NON-NLS-1$
			if (this._message) {
				this._details.appendChild(this._message);
			} else if (this._descriptions && (this._descriptions.title || this._descriptions.description)) {
				if (this._descriptions.description) {
					dojo.empty(this._details);
					this._populateDetails(this._descriptions.description);
				}
				if (this._descriptions.title) {
					var link = document.createElement("a"); //$NON-NLS-1$
					link.href = this._descriptions.url;
					link.appendChild(document.createTextNode(this._descriptions.title));
					this._resourceLink = new ResourceLink({
						'presentationUri': this._descriptions.url, //$NON-NLS-1$
						'lazyFetch': true //$NON-NLS-1$
					}, link);
					dojo.empty(this._summary);
					this._summary.appendChild(link);
				}
			} else {
				if (this.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
					this._details.appendChild(document.createTextNode(this.messages.noDescTeam));
				} else {
					this._details.appendChild(document.createTextNode(this.messages.noDescProject));
				}
				
			}
			if (!this._summary.firstChild) {
				dojo.addClass(this._summary, "hidden"); //$NON-NLS-1$
			}
			if (!this._details.firstChild) {
				dojo.addClass(this._details, "hidden"); //$NON-NLS-1$
			}
		},
		
		_populateDetails: function(/* string */input) {
			input = dojo.trim(input.replace("\r", "")); //$NON-NLS-1$ //$NON-NLS-2$
			for (var i = input.indexOf("\n"); i != -1; i = input.indexOf("\n")) { //$NON-NLS-1$ //$NON-NLS-2$
				this._details.appendChild(document.createTextNode(input.substring(0, i)));
				this._details.appendChild(document.createElement("br")); //$NON-NLS-1$
				input = input.substring(i + 1);
			}
			this._details.appendChild(document.createTextNode(input));
		},
		
		/*
		 * Shows a message (a DOM node) in the viewlet content area, replacing any other
		 * content that was there.
		 */
		_showMessage: function(/* Node */node) {
			this._message = node;
			this.update();
		},
		
		settingsChanged: function(/* object */oldSettings, /* object */ newSettings) {
			if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
				this.getSite().refresh();
			} else {
				this.update();
			}
		},
		
		_createNoScopeMsg: function() {
			var msgSpan = document.createElement("span"); //$NON-NLS-1$
			msgSpan.innerHTML = this.messages.notScoped;
			if (this.getSite().isDashboardEditing()) {
				var editSpan = document.createElement("span"); //$NON-NLS-1$
				var nodes = I18nUtil.addMessage(editSpan, this.messages.notScopedEdit);
				this.connect(nodes[0], "onclick", this._handleToggleSettings); //$NON-NLS-1$
				msgSpan.appendChild(editSpan);
			}
			return msgSpan;
		},
		
		_handleToggleSettings: function(event) {
			dojo.stopEvent(event);
			this.getSite().toggleSettings();
		},
		
		uninitialize: function() {
			if (this._resourceLink) {
				this._resourceLink.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
	
})();
