/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportTemplatesNavigator");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.app.context");

dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.FolderWidget");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;

function deleteTemplate(e, itemId, messages, controller) {
	e.preventDefault();
	var handler = {
		success: function() {
		},
		error: function(e) {
			alert(e.message);
		}
	}
	if (!confirm(messages.confirmDelTemplate)) {
		return;
	}
	var responseHandler = new ServiceResponseHandler(handler, "success", "error");
	ReportsClient.postDeleteReportDescriptor(responseHandler, {reportUUID: itemId});
}

(function() { // module container

var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;

var DROPDOWN_HOVER = ImagePool.DD_ARROW_HOV;
var DROPDOWN       = ImagePool.DD_ARROW;

var TEMPLATE_PARTIAL_VIEW_COUNT = 5;

dojo.declare("com.ibm.team.reports.web.ui.internal.ReportTemplatesNavigator", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportTemplatesNavigator.html"),
	
	CREATE_NEW_TEMPLATE_ICON_CLASS: "newTemplateIcon",
	FIND_NEW_TEMPLATES_ICON_CLASS: "findNewTemplatesIcon",
		
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
	},
	
	initializePulldown: function(navSection) {
		var span = navSection._togglerLink.childNodes[2];
		navSection._togglerLink.removeChild(span);
		
		var table = document.createElement("table");
		table.width = "100%";
		table.cellPadding = "0";
		table.cellSpacing = "0";
		table.className = "compactTable";
		var tbody = document.createElement("tbody");
		table.appendChild(tbody);
		var tr = document.createElement("tr");
		tbody.appendChild(tr);
		var td1 = document.createElement("td");
		td1.appendChild(span);
		tr.appendChild(td1);
		var td = document.createElement("td");
		td.align = "right";
		tr.appendChild(td);
		
		var a = document.createElement("a");
		a.href = "#";
		a.dojoAttachPoint = "_dropDown";
		a.className = "dropDownFloatingLink";
		// Hide the drop-down indicator initially
		a.style.visibility = "hidden";
		this._dropDown = a;
		
		var img = document.createElement("img");
		img.src = DROPDOWN;
		img.dojoAttachPoint = "_dropDownImage";
		img.className = "reportNavIcon";
		this._dropDownImage = img;
		
		a.appendChild(img);
		navSection._togglerLink.appendChild(a);
		td.appendChild(a);
		navSection._togglerLink.appendChild(table);
		
		// Hook the events for the drop-down indicator
		this.connect(img, "onmouseover", this._imgMouseOverHandler);
		this.connect(img, "onmouseout", this._imgMouseOutHandler);
		this.connect(img, "onclick", this._imgClickHandler);

		this.connect(navSection._togglerLink, "onmouseover", this._togglerLinkMouseOverHandler);
		this.connect(navSection._togglerLink, "onmouseout", this._togglerLinkMouseOutHandler);
	},
	
	setNavigationSection: function(navSection) {
		this.navSection = navSection;
	},
	
	setReportTemplates: function(reportDescriptors) {
		this.removeTemplates();
		
		this._templateLinks = new Array();
		this._hidePartialLinks = new Array();
		
		this.navSection.setVisible(true);
		
		// Sort them first.
		if (reportDescriptors && reportDescriptors.length > 0) {
			var sorted = this._sort(reportDescriptors);
			for (var i = 0; i < sorted.length; i++) {
				this._templateLinks[i] = this._addTemplate(sorted[i]);
				if (i >= TEMPLATE_PARTIAL_VIEW_COUNT) {
					this._hidePartialLinks.push(this._templateLinks[i]);
					dojo.addClass(this._templateLinks[i], "com-ibm-team-reports-web-nav-hidden");
				}
			}
		}
		
		this.navSection.update();	
	},
	
	removeTemplates: function() {
		if (this._templateLinks) {
			for (var i = 0; i < this._templateLinks.length; i++) {
				if (this._templateLinks[i].parentNode)
					this._templateLinks[i].parentNode.removeChild(this._templateLinks[i]);
			}
		}
	},
	
	_addTemplate: function(template) {
		// Construct the link
		var href = "#action=com.ibm.team.reports.newReport&reportUUID=" + template.reportUUID;
		href += "&name=" + template.name;
		var templateLink = this.navSection.addSectionRow(template.name, href, ImagePool.REPORTTEMPLATE_OBJ);

		// Add actions
		var messages = this.messages;
		var controller = this.controller;
		var editQueryURL = "#action=com.ibm.team.reports.editTemplate&reportUUID=" + template.reportUUID;
		this.navSection.addArtifactCommand(templateLink, this.messages.deleteTemplate, function(e) { deleteTemplate(e, template.reportUUID, messages, controller)}, "delete-report-template-command", true);
		this.navSection.addArtifactCommand(templateLink, this.messages.editTemplate, editQueryURL, "edit-report-template-command", true);
		return templateLink;
	},		

	/**
	 * Helper function to sort a list of named items
	 * Note: Assume that names are unique
	 */
	_sort: function(items) {
		var names = [];
		for (var i = 0; i < items.length; i++) {
			names[i] = items[i].name;
		}
		names = names.sort();
		var result = [];
		for (var i = 0; i < names.length; i++) {
			result[i] = this._find(names[i], items);
		}
		return result;
	},
	
	_find: function(name, items) {
		for (var i = 0; i < items.length; i++) {
			if (items[i].name == name) {
				return items[i];
			}
		}
		return null;
	},		
			
	_remoteError: function(error) {
		alert(error.message);
	},
	
	hasShowPartial: function() {
		return this._hidePartialLinks ? this._hidePartialLinks.length > 0 : false;
	},
	
	onShowPartial: function() {
		if (this._hidePartialLinks) {
			for (var i = 0; i < this._hidePartialLinks.length; i++) {
				dojo.addClass(this._hidePartialLinks[i],"com-ibm-team-reports-web-nav-hidden");
			}
		}
	},
	
	onMaximize: function() {
		if (this._hidePartialLinks) {
			for (var i = 0; i < this._hidePartialLinks.length; i++) {
				dojo.removeClass(this._hidePartialLinks[i],"com-ibm-team-reports-web-nav-hidden");
			}
		}
	},
	
	_imgMouseOverHandler: function(e) {
		e.preventDefault();
		this._dropDownImage.src = DROPDOWN_HOVER;
	},
	
	_imgMouseOutHandler: function(e) {
		e.preventDefault();
		this._dropDownImage.src = DROPDOWN;
	},
	
	_imgClickHandler: function(e) {
		e.preventDefault();
		this._openMenu(e);
		dojo.stopEvent(e);
	},
	
	_togglerLinkMouseOverHandler: function(e) {
		e.preventDefault();
		if (this._menu != null) {
			// We're already in the menu, ignore this
			this._mouseOutOccurred = false;
			return;
		}
		this._dropDown.style.visibility = "visible";
	},
	
	_togglerLinkMouseOutHandler: function(e) {
		e.preventDefault();
		if (this._menu != null) {
			// We're in the menu, don't remove the hover
			// but remember that we left
			this._mouseOutOccurred = true;
			return;
		}
		this._dropDown.style.visibility = "hidden";
	},
	
	_openMenu: function(e) {
		if (this._menu) {
			this._closeMenu();
			return;
		}
		var menu = new dijit.Menu();
		menu.addChild(new dijit.MenuItem({label: this.messages.newTemplateDotDotDot, iconClass: this.CREATE_NEW_TEMPLATE_ICON_CLASS, onClick: dojo.hitch(this,this._newTemplate)}));
		menu.addChild(new dijit.MenuItem({label: this.messages.findTemplatesDotDotDot, iconClass: this.FIND_NEW_TEMPLATES_ICON_CLASS, onClick: dojo.hitch(this,this._deployNewTemplates)}));
		var coords = dojo.coords(e.target, true);
		dijit.popup.open({ popup: menu, parent: this, x: coords.x + 10, y: coords.y + 10, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
		//register event handler that causes menu to close
		this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
		this._menu = menu;
		menu.focus();			
	},
	
	_closeMenu: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		if (this._mouseOutOccurred) {
			this._dropDown.style.visibility = "hidden";
		}
		this.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._menu = null;
	},
	
	_newTemplate: function() {
		document.location.href = "#action=com.ibm.team.reports.editTemplate";
	},
	
	_deployNewTemplates: function() {
		// Prompt the user to select a project area or team area
		var projectArea = jazz.app.context.get();
		var self = this;
		var params = {
			projects: true,
			teams: true,
			allowsMultiple: false,
			selection: projectArea.itemId,
			filterProjectArea: projectArea.itemId,
			onOk: function(selectedItem, action) {
				self._onProcessOk(selectedItem);
			}
		};
		new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
	},
	
	_onProcessOk: function(selectedItem) {
		// Load the deployNewTemplates page
		document.location.href = "#action=com.ibm.team.reports.deployNewTemplates&processAreaUUID=" + selectedItem.itemId;
	}
});

})();