/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.ui.internal.dialogs.ExportDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.internal.UIHelper"); //$NON-NLS-1$ 
dojo.requireLocalization("com.ibm.team.process.authoring.web.ui.internal.dialogs", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	
(function() {

	var _currentRequest;
	var _WIDTH_IN_EM = 50;
	var processExportKey;
	
	dojo.declare("com.ibm.team.process.authoring.web.ui.internal.dialogs.ExportDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web", "ui/internal/dialogs/templates/ExportDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
		widgetsInTemplate: true,
		constructor: function(args) {
			if (!args) {
				args = {};
			}
			this.width = _WIDTH_IN_EM + "em"; //$NON-NLS-1$
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.ui.internal.dialogs", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			var self = this;
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.messages.ExportDialog_1,
				width: this.width,
				destroyContents: true,
				onClose: function() {
					if (self.processExportKey) {
						self._invokeService(self.serviceUrl + "/cancelExport?exportKey=" + self.processExportKey, null, null); //$NON-NLS-1$
					}
				}
			});
			this._topText.innerHTML = this.message;
			
			this.okButton.disabled = true;
			//TODO: disable Cancel button for now until cancel support is implemented
			this.cancelButton.disabled = true;
			
			var successCallback = function(response) {
				var jsonResponse = null;
				try {
					jsonResponse = dojo.fromJson(response);
				} catch (e) {
					//
				}
				
				if (jsonResponse && jsonResponse.message) {
					self.cancelButton.disabled = false;
					self._setStatusMessage(jsonResponse.message, true);
				} else {
					self.dialog.close();
					self.exportCallback(self.serviceUrl);
				}
			};
			
			var errorCallback = function(error) {
				var responseText = "";//$NON-NLS-1$
				if (error.responseText) {
					responseText = error.responseText;
				} else if (error.message) {
					responseText = error.message;
				} else {
					responseText = error;
				}
				
				self._setStatusMessage(responseText, true);
				
			};
			
			this._invokeService(this.serviceUrl, successCallback, errorCallback); //$NON-NLS-1$
			try {
				com.ibm.team.process.authoring.web.common.internal.UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			} catch (e) {
				//failed to resize on IE for tooltipDialog, have to catch it. This is logged in workitem 111790
			}
			
			this.connect(this.cancelButton, "onclick", function(e) { //$NON-NLS-1$
				dojo.stopEvent(e);
				if (this.intervalId) {
					clearInterval(this.intervalId);
				}
				if (_currentRequest) {
					_currentRequest.cancel();
				}
				if (window.stop) {
					window.stop();
				}
				
				this.dialog.close();
			});
			this.connect(this.okButton, "onclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$ 
		},
		
		exportCallback: function(serviceUrl) {
			window.location = serviceUrl;
		},
		
		_onOk: function(e) {
			this.okButton.disabled = true;
			if (e) {
				dojo.stopEvent(e);
			}
			
			this.processExportKey = null;
			this.dialog.close();
			
		},
		
		_clearStatusMessage: function() {
			this.status.style.visibility = "hidden";//$NON-NLS-1$
		},
		
		_setStatusMessage: function(msg, isError) {
			this.status.style.visibility = "visible";//$NON-NLS-1$
			dojox.xml.parser.replaceChildren(this.status, document.createTextNode(msg));
			this.status.className = isError ? "ErrorStatus" : "NormalStatus";//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_invokeService: function(url, successCallback, errorCallback) {
			var args = {
				url: url,
				headers: {
					Accept: "*" //$NON-NLS-1$
				}
			};
			var deferred = jazz.client.xhrGet(args);
			
			deferred.addCallback(successCallback);
			deferred.addErrback(errorCallback);
			
			this._currentRequest = deferred;
		}
		
	});
	
})();
