/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.ui.internal.dialogs.ImportDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("dojo.io.iframe"); //$NON-NLS-1$
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
dojo.require("dojox.uuid"); //$NON-NLS-1$
dojo.require("dojox.uuid.generateTimeBasedUuid"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.authoring.web.ui.internal.dialogs", "Messages");//$NON-NLS-1$//$NON-NLS-2$
dojo.require("com.ibm.team.process.authoring.web.common.internal.UIHelper");//$NON-NLS-1$
(function() {

	var Messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.ui.internal.dialogs", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
	var DEFAULT_INTRO_TEXT = Messages.ImportDialog_1;
	var UIHelper = com.ibm.team.process.authoring.web.common.internal.UIHelper;
	var SERVICE_URL = UIHelper.getProcessAuthoringRoot() + "/file"; //$NON-NLS-1$
	var UPLOAD_PROGRESS_SERVICE_URL = dojo.baseUrl + "../../service/com.ibm.team.process.internal.service.web.IProcessDefinitionFileService/checkUploadProgress";//$NON-NLS-1$
	var _uploadFileInput_onchange_handle;
	var _uploadFileInput;
	var _okButton_onclick_handle;
	var _jsonResponse;
	var _currentRequest;
	var _WIDTH_IN_EM = 50;
	
	dojo.declare("com.ibm.team.process.authoring.web.ui.internal.dialogs.ImportDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web", "ui/internal/dialogs/templates/ImportDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
		widgetsInTemplate: true,
		constructor: function(args) {
			if (!args) {
				args = {};
			}
			//for private practice
			if (args.processDescriptionUrl) {
				this.processDescriptionUrl = args.processDescriptionUrl;
			}
			//for private process description
			if (args.processDescriptionOwnerUrl) {
				this.processDescriptionOwnerUrl = args.processDescriptionOwnerUrl;
				this.processDescriptionOwnerName = args.processDescriptionOwnerName;
			}
			//for shared resources
			if(args.isSharedProcessDescriptionImporting){
				this.isSharedProcessDescriptionImporting = args.isSharedProcessDescriptionImporting;
			}
			this.width = _WIDTH_IN_EM + "em"; //$NON-NLS-1$
			this.uploadId = null;
			this.intervalId = null;
			var properties = {
				"width": this.width //$NON-NLS-1$
			};
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.ui.internal.dialogs", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.messages.ImportDialog_2,
				width: this.width,
				destroyContents: true
			});
			this._topText.innerHTML = DEFAULT_INTRO_TEXT;
			
			var div = this._uploadInputDiv;
			dojo.empty(div);
			var input1 = document.createElement("input"); //$NON-NLS-1$
			input1.type = "file"; //$NON-NLS-1$
			input1.name = "uploadFileInput";//$NON-NLS-1$
			input1.size = _WIDTH_IN_EM - 5; //size is the width of the input in characters
			this._uploadFileInput = input1;
			this._uploadFileInput_onchange_handle = dojo.connect(this._uploadFileInput, "onchange", this, "_handleDirectoryChange"); //$NON-NLS-1$ //$NON-NLS-2$
			if (!dojo.isIE) {
				// setting focus here causes IE to not show the field, so don't 
				//do it 
				this._uploadFileInput.focus();
			}
			div.appendChild(input1);
			
			try {
				com.ibm.team.process.authoring.web.common.internal.UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton], -1, 3);
			} catch (e) {
				//failed to resize on IE for tooltipDialog, have to catch it. This is logged in workitem 111790
			}
			
			this.connect(this.cancelButton, "onclick", "onCancel");  //$NON-NLS-1$ //$NON-NLS-2$ 
			this._okButton_onclick_handle = this.connect(this.okButton, "onclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$ 
		},
		/**
		 * enable the OK button if the file is a zip
		 * @param e is an event
		 */
		_handleDirectoryChange: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			
			this._clearStatusMessage();
			var fullPath = this._uploadFileInput.value;
			if (fullPath.length !== 0) {
				var parts = fullPath.split("\\"); //$NON-NLS-1$
				var fileName = parts[parts.length - 1];
				var extensionIndex = fileName.toLocaleLowerCase().lastIndexOf(".zip"); //$NON-NLS-1$
				if (!(extensionIndex == fileName.length - 4)) {
					this._setStatusMessage(this.messages.ImportDialog_3, true);
					return;
				} else {
					this.okButton.disabled = false;
				}
			}
		},
		_runSelect: function(e) {
			dojo.stopEvent(e);
			
		},
		_runDeselect: function(e) {
			dojo.stopEvent(e);
			
		},
		_onOk: function(e) {
			if (e) {
				dojo.stopEvent(e);
			}
			this._setStatusMessage(this.messages.ImportDialog_4);
			this._showProgressBar(false);
			
			// create unique id for this upload
			this.uploadId = dojox.uuid.generateTimeBasedUuid();
			
			var uploadUrl;
			if (this.processDescriptionOwnerUrl) {//import private process description
				uploadUrl = SERVICE_URL + "?uploadId=" + this.uploadId + "&ownerUrl=" + this.processDescriptionOwnerUrl; //$NON-NLS-1$ //$NON-NLS-2$
			} else if (this.processDescriptionUrl) {//import private practice
				uploadUrl = SERVICE_URL + "?uploadId=" + this.uploadId + "&ownerUrl=" + this.processDescriptionUrl; //$NON-NLS-1$ //$NON-NLS-2$
			} else { //import shared resources
				uploadUrl = SERVICE_URL + "?uploadId=" + this.uploadId + "&isSharedProcessDescriptionImporting=" + this.isSharedProcessDescriptionImporting; //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			_currentRequest = dojo.io.iframe.send({
				form: this._uploadForm,
				content: {
					"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID") //$NON-NLS-1$ //$NON-NLS-2$
				},
				handleAs: "json",//$NON-NLS-1$
				url: uploadUrl,
				load: dojo.hitch(this, function(response, ioArgs) {
					this._handleLoad(response, ioArgs);
				}),
				error: dojo.hitch(this, function(response, ioArgs) {
					this._handleErrorOnUpload(response, ioArgs);
				})
			});
			
			this.okButton.disabled = true;
			this._uploadFileInput.disabled = true;
			
			//TODO: show import progress
			//this.intervalId = setInterval(dojo.hitch(this, "_pollUploadProgress"), 1000); //$NON-NLS-1$
		},
		
		onCancel: function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			if (this.intervalId) {
				clearInterval(this.intervalId);
			}
			if (_currentRequest) {
				_currentRequest.cancel();
			}
			if (window.stop) {
				window.stop();
			}
			this.dialog.close();
		},
		
		_pollUploadProgress: function() {
			dojo.xhrGet({
				url: UPLOAD_PROGRESS_SERVICE_URL,
				handleAs: "json", //$NON-NLS-1$
				content: {
					uploadId: this.uploadId
				},
				preventCache: true,
				load: dojo.hitch(this, function(response, ioArgs) {
					this._progress.update({
						maximum: response.size,
						progress: response.uploadedBytes
					});
					if (response.size == response.uploadedBytes) {
						clearInterval(this.intervalId);
					}
					return response;
				}),
				error: function(response, ioArgs) {
					return response;
				}
			});
		},
		
		_handleLoad: function(response, ioArgs) {
			clearInterval(this.intervalId);
			if (response instanceof Object) {
				this._jsonResponse = response;
			} else {
				this._jsonResponse = dojo.fromJson(response);
			}
			switch (this._jsonResponse.status) {
				case 200:
					//success case
					this.onSuccess(this._jsonResponse);
					break;
				default:
					//any other error
					this._setStatusMessage(this._jsonResponse.message, true);
					this._progress.domNode.style.display = "none"; //$NON-NLS-1$
					this._uploadFileInput.disabled = false;
					//enable the OK button so they can fix the problem and try again
					this.okButton.disabled = false;
					break;
			}
		},
		
		onSuccess: function(jsonResponse) {
			this._hideFileUploadSection();
			_currentRequest = null;
			this.okButton.style.display = "none"; //$NON-NLS-1$
			this.cancelButton.innerHTML = this.messages.ImportDialog_17;
			
			// show import summary
			this._topText.innerHTML = jsonResponse.message;
			
			//			this.dialog.close();
		},
		
		_handleErrorOnUpload: function(response, ioArgs) {
			if (!response.dojoType === "cancel") { //$NON-NLS-1$
				clearInterval(this.intervalId);
				this._setStatusMessage(response, true);
			}
		},
		
		_hideFileUploadSection: function() {
			this._clearStatusMessage();
			this._uploadForm.style.display = "none"; //$NON-NLS-1$
			this._progress.domNode.style.display = "none"; //$NON-NLS-1$
		},
		
		_showFileUploadSection: function() {
			this._uploadForm.style.display = "block";//$NON-NLS-1$
		},
		
		_clearStatusMessage: function() {
			this.status.style.visibility = "hidden";//$NON-NLS-1$
		},
		
		_setStatusMessage: function(msg, isError) {
			this.status.style.visibility = "visible";//$NON-NLS-1$
			dojox.xml.parser.replaceChildren(this.status, document.createTextNode(msg));
			this.status.className = isError ? "ErrorStatus" : "NormalStatus";//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_showProgressBar: function(indeterminate) {
			this._progress.update({
				indeterminate: indeterminate
			});
			this._progress.domNode.style.display = "block";//$NON-NLS-1$
		},
		
		uninitialize: function() {
			if (this._uploadFileInput_onchange_handle) {
				dojo.disconnect(this._uploadFileInput_onchange_handle);
			}
			this.inherited(arguments);
		}
	});
	
})();
