/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.ui.internal.editors.AttachmentPickerDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.ui.internal.editors.AbstractResourcePickerDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.internal.UIHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	var AbstractResourcePickerDialog = com.ibm.team.process.authoring.web.ui.internal.editors.AbstractResourcePickerDialog;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants;
	
	dojo.declare("com.ibm.team.process.authoring.web.ui.internal.editors.AttachmentPickerDialog", [AbstractResourcePickerDialog, dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web", "ui/internal/editors/templates/AttachmentPickerDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		setupObjects: function() {
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.dialogTitle,
				destroyContents: true,
				width: this.width,
				modal: this.modal
			});
		},
		workingCopy: null,
		
		constructor: function(args) {
		
			this.setProperties({
				"modal": true, //$NON-NLS-1$ 
				"dialogTitle": this.messages.AttachmentPickerDialog_5, //$NON-NLS-1$
				"selectionTitleLabel": this.messages.AttachmentPickerDialog_4, //$NON-NLS-1$
				"allowMultiple": true, //$NON-NLS-1$
				"disableAfterSelection": false, //$NON-NLS-1$
				"AddCloseButtonLabel": this.messages.AttachmentPickerDialog_2, //$NON-NLS-1$
				"CloseButtonLabel": this.messages.AttachmentPickerDialog_3 //$NON-NLS-1$
			}, args);
			
		},
		
		_addOwnedPractices: function (sharedPracticeUrls) {
			for (i = 0; i < this.workingCopy.practiceList.length; i++) {
				if (dojo.indexOf(sharedPracticeUrls, this.workingCopy.practiceList[i].resource) == -1) {
					this.practices.push({
						name: this.workingCopy.practiceList[i].title,
						url: this.workingCopy.practiceList[i].resource
					});
					this.resourceGroups.push({
						'type': this.workingCopy.practiceList[i].title, //$NON-NLS-1$
						'list': [] //$NON-NLS-1$
					});
				}
			}
		},
		
		postCreate: function() {
			this.resourceGroups = [];
			ProcessAuthoringClient.getAllSharedPractices({
				'load': dojo.hitch(this, function(data) { //$NON-NLS-1$
					this.practices = this.loadAllSharedPractices(data);
					var sharedPracticeUrls = [];
					for (var i = 0; i < this.practices.length; i++) {
						this.resourceGroups.push({
							'type': this.practices[i].name, //$NON-NLS-1$
							'list': [] //$NON-NLS-1$
						});
						sharedPracticeUrls.push(this.practices[i].url);
					}
					
					//added private practice within process description
					this._addOwnedPractices(sharedPracticeUrls);
					
					this.init();
				}),
				error: dojo.hitch(this, function(data, ioargs) {
					if(ioargs.xhr.status == 403) {
						// user is not allowed to access shared practices
						
						this.practices = [];
						//added private practice within process description
						this._addOwnedPractices([]);
					
						this.init();
					} else {
						alert(data);
					}
				})
			});
		},
		
		
		_onAddClose: function(e) {
			this._onAdd(e);
			this.dialog.close();
		},
		
		_onCancel: function(e) {
			this.dialog.close();
		},
		
		onClose: function() {
			dojo.disconnect(this.keyStrokeFilter);
		},
		
		setupButtons: function() {
			this.addCloseButton.appendChild(document.createTextNode(this.AddCloseButtonLabel));
			this.closeButton.appendChild(document.createTextNode(this.CloseButtonLabel));
			this.connect(this.closeButton, "onclick", this._onCancel); //$NON-NLS-1$
			dojo.style(this.addCloseButton, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(this.closeButton, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
			try {
				UIHelper.resizeI18NButtons(this.dialogFooter, [this.addCloseButton, this.closeButton], -1, 3);
			} catch (e) {
				//failed to resize on IE for tooltipDialog, have to catch it. This is logged in workitem 111790
			}
		},
		
		loadAllSharedPractices: function(data) {
			var children = data.childNodes;
			var practices = UIHelper.getNode(children, "Practices", ProcessAuthoringConstants.NAMESPACE_JTP); //$NON-NLS-1$
			var sharedPracticeList = [];
			if (practices) {
				var allPractices = UIHelper.getNodes(practices.childNodes, ProcessAuthoringConstants.XMLELEMENT_PRACTICE_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
				dojo.forEach(allPractices, function(practice) {
					var practiceResource = UIHelper.getResourceUrl(practice);
					var practiceName = UIHelper.getNodeTitle(practice);
					if (practiceResource && practiceName) { //$NON-NLS-1$ //we only display the practice that has about attribute and title
						sharedPracticeList.push({
							name: practiceName,
							url: practiceResource
						});
					}
				}, this);
				sharedPracticeList.sort(function(practice1, practice2) {
					return practice1.name.toLocaleLowerCase().localeCompare(practice2.name.toLocaleLowerCase());
				});
			}
			return sharedPracticeList;
		},
		
		loadAttachmentsByPractice: function(data, idx) {
			var attachments = UIHelper.getAttachmentNodes(data);
			var attachmentList = [];
			dojo.forEach(attachments, function(attachment) {
				var path = UIHelper.getAttachmentPath(attachment);
				var resource = UIHelper.getResourceUrl(attachment);
				var content = UIHelper.getAttachmentContent(attachment);
				
				attachmentList.push({
					name: path,
					url: content,
					resourceUrl: resource
				});
			}, this);
			attachmentList.sort(function(attachment1, attachment2) {
				return attachment1.name.toLocaleLowerCase().localeCompare(attachment2.name.toLocaleLowerCase());
			});
			this.resourceGroups[idx].list = attachmentList;
		},
		
		_getResources: function(idx) {
			var practiceResource = this.practices[idx].url;
			
			ProcessAuthoringClient.getAttachmentsByPractice({
				'url': practiceResource, //$NON-NLS-1$
				'load': dojo.hitch(this, function(data) { //$NON-NLS-1$
					this.loadAttachmentsByPractice(data, idx);
					this.loadResources(this.resourceGroups[idx].list);
					this._clearStatusMessage();
				})
			});
		}
	});
})();
