/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.ui.internal.editors.PracticeTreePane"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	dojo.declare("com.ibm.team.process.authoring.web.ui.internal.editors.PracticeTreePane", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web", "ui/internal/editors/templates/PracticeTreePane.html"), //$NON-NLS-1$ //$NON-NLS-2$
		trees: null,
		editPractice: false,
		viewOnly: false,
		isOutlinePane: false,
		isPlainTree: false,
		rootOnly: false,
		isAssociated: false,
		editToolBar: false,
		selectedNodeItem: null,
		selectedNodeItemIndex: null,
		controller: null,
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			this.trees = [];
		},
		
		addPracticeTree: function(practiceTree) {
		    var needplus = false;
			for (var i = 0; i < this.trees.length; i++) {
				if (practiceTree.indexInParent == this.trees[i].indexInParent) {
					needplus = true;
				}
				if (needplus) {
					this.trees[i].indexInParent = this.trees[i].indexInParent + 1;
				}
			}
			this.trees.push(practiceTree);
			//sort the trees alphabetically
			if (this.trees.length === 1) {
				this.treeSection.appendChild(practiceTree.domNode);
			} else {
				this.trees.sort(function(a, b) {
					return a.indexInParent - b.indexInParent;
				});
				
				var index = dojo.indexOf(this.trees, practiceTree);
				var next = this.trees[index + 1];
				if (next) {
					dojo.place(practiceTree.domNode, next.domNode, "before"); //$NON-NLS-1$	
				} else {
					this.treeSection.appendChild(practiceTree.domNode);
				}
			}
		},
		
		removePracticeTrees: function() {
			dojo.empty(this.treeSection);
			this.trees = [];
		},
		
		removePracticeTree: function(practiceResource) {
			var practiceId = practiceResource.substring(practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
			var treePracticeId = null;
			var treeNode = null;
			dojo.some(this.trees, function(tree) {
				treePracticeId = tree.practiceResource.substring(tree.practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
				if (tree.practiceResource === practiceResource || treePracticeId === practiceId) {
					treeNode = tree;
					return true;
				}
				return false;
			});
			this.trees.splice(dojo.indexOf(this.trees, treeNode), 1);
			treeNode.destroyRecursive();
		},
		
		getItem: function(practiceResource, node, callback) {
			var practiceId = practiceResource.substring(practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
			var treePracticeId = null;
			dojo.some(this.trees, function(tree) {
				treePracticeId = tree.practiceResource.substring(tree.practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
				if ((tree.practiceResource === practiceResource) || (treePracticeId === practiceId)) {
					if (node) {
						tree.model.store.fetchItemByIdentity({
							identity: node.id,
							onItem: function(item) {
								callback(item, tree);
							}
						});
					} else {
						// return root item
						callback(tree.rootNode.item, tree);
					}
					return true;
				}
				return false;
			}, this);
			
		},
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////
		//  Make sure selectTree is called in the following conditions
		//  1. Use PracticeTree.attr("selectedItem", path) to select a ROOT node in the practice tree.
		//  2. Use PracticeTree.attr("path", path) to select a non-ROOT node in the practice tree.
		//  3. Perform a click event in PracticeTree.onClick.
		//  4. Add a practice/practice element/category in left navigator.
		//////////////////////////////////////////////////////////////////////////////////////////////////////
		selectTree: function(practiceTree) {
			dojo.forEach(this.trees,function(tree) {
				if (tree === practiceTree) {
					dojo.toggleClass(tree.domNode, "selectedTree", true); //$NON-NLS-1$
				} else {
					dojo.toggleClass(tree.domNode, "selectedTree", false); //$NON-NLS-1$
				}
			});
		},
		
		uninitialize: function() {
			if (this.editToolBar) {
				this.editToolBar.destroyRecursive();
			}
			dojo.forEach(this.trees, function(tree) {
				tree.destroyRecursive();
			});
			
			this.inherited(arguments);
		}
	});
	
})();
