/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.jfs.web.ui.internal.editors.ProcessDescriptionEditorGadget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.ui.internal.editors.ProcessDescriptionEditorGadget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.internal.ProcessAuthoringConstants;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.client.internal.ProcessAuthoringClient;
	
	dojo.declare("com.ibm.team.process.authoring.jfs.web.ui.internal.editors.ProcessDescriptionEditorGadget", [com.ibm.team.process.authoring.web.ui.internal.editors.ProcessDescriptionEditorGadget], { //$NON-NLS-1$ 
		_initialize: function(args) {
			this.inherited(arguments);
			var prefs = new gadgets.Prefs();
			this.sharedSupport = "true" === prefs.getString(ProcessAuthoringConstants.PROCESS_AUTHORING_READ_SHARED_SUPPORT); //$NON-NLS-1$
			this.canWriteShared = "true" === prefs.getString(ProcessAuthoringConstants.PROCESS_AUTHORING_WRITE_SHARED_SUPPORT); //$NON-NLS-1$
			this.viewOnly = this.isShared; //shared process description is view only in the foundation
			this.workingCopy.currentLanguage = prefs.getString(ProcessAuthoringConstants.PREFS_PROCESS_LOCALE);
		},
		
		loadProcessDescription: function() {
			var self = this;
			ProcessAuthoringClient.getProcessDescription({
				lastModified: this.workingCopy.lastModified,
				resourceURL: this.resourceURL,
				load: function(data, ioArgs) {
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_PERMISSTION_CHECK_DONE, [true]);
					self.processDescriptionETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
					self.populateData(data);
					self.resizeEditor();
				},
				error: function(error) {
					if (error.responseText && error.status == ProcessAuthoringConstants.SC_FORBIDDEN) {
						error.message = error.responseText;
						error.headerMessage = self.messages.ProcessDescriptionError_3;
						self.displayErrorMessage(error);
					} else {
						self.displayErrorMessage(error);
					}
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_PERMISSTION_CHECK_DONE, [false]);
				}
			});
		},
		
		_hide: function(isReadable) {
			if (isReadable) {
				this.primaryFieldContainer.style.display = "block"; //$NON-NLS-1$
				this.detailWrapper.style.display = "block"; //$NON-NLS-1$
				dojo.toggleClass(this.contentWrapper, ProcessAuthoringConstants.CSS_HIDE, false);
			}
		},
		
		loadData: function() {
			if (this.isRefreshCalled) {
				this.loadProcessDescription();
			}
		},
		
		postCreate: function() {
			if (!this.isNew) {
				this.checkPermissionDoneHandle = dojo.subscribe(ProcessAuthoringConstants.PUBLISHTYPE_PERMISSTION_CHECK_DONE, this, this._hide);
				this.loadProcessDescription();
			} else {
				this._hide(true);
			}
			this.inherited(arguments);
		},
		
		toggleDirty: function(isDirty) {
			gadgets.rpc.call("", ProcessAuthoringConstants.RPCSERVICE_PROCESSDESCRIPTION_TOGGLEDIRTY, null, isDirty, this.name.value); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		uninitialize: function() {
		
			if (this.checkPermissionDoneHandle) {
				dojo.unsubscribe(this.checkPermissionDoneHandle);
				this.checkPermissionDoneHandle = null;
			}
			this.inherited(arguments);
		}
	});
})();
